/*
 *
 * \attention  This program is provided as is. You can redistribute it and/or
 * modify it. Fujitsu Semiconductor Limited accepts no responsibility or
 * liability for any errors or omissions.
 *
 * Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
 */
/*
 CFDL module skel_drv
 ALI (Only RTOS use it.)
*/

#ifndef __LINUX__

#include "fapex.h"
#include "fapex/abst_lib.h"
#include "fapex/abst_types.h"
#include "fapex_cfdl.h"
#include "fapex_cfdl_priv.h"

/* prototype */
extern void* FAPEXCFDL_GetCfdlHdl(void);

/*!
*******************************************************************************
**
** \brief  An ALI function (open method)
**         This function is called with the systemcall "open".
**
** \param  id     The device identifier.
** \param  arg2   Reserved.
**                This parameter indicate a open mode (read only/write only/read-write).
**                This implementation supports only read-write mode.
**
******************************************************************************/
FAPEXOPENRET_T FAPEXSAL_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T arg2)
{
    FAPEXCFDL_StrT  *pCoreHdl;
    FAPEXOPENRET_T  ret;

    if((id < 0) || (id >= FAPEX_MAXIMUM_MINOR)) {
        errno = -FAPEX_SYS_ERR_INVALID_STATUS;
        return -1;
    }

    pCoreHdl = (FAPEXCFDL_StrT*)FAPEXCFDL_GetCfdlHdl();

    if(pCoreHdl->fop[id]->open != NULL) {
        ret = pCoreHdl->fop[id]->open( id,
                                       (FAPEXFDLOPENARG2_T)(pCoreHdl->pOsPrivate[id]) );
        if(ret < 0) {
            errno = -ret;
            return -1;
        }
    }
    return (FAPEXOPENRET_T)id;
}

/*!
*******************************************************************************
**
** \brief  An ALI function (release method)
**         This function is called with the systemcall "close".
**
** \param  id     The device identifier.
**
******************************************************************************/
FAPEXRELRET_T FAPEXSAL_Release(FAPEXRELARG1_T id)
{
    FAPEXCFDL_StrT  *pCoreHdl;
    FAPEXRELRET_T   ret;

    if((id < 0) || (id >= FAPEX_MAXIMUM_MINOR)) {
        errno = -FAPEX_SYS_ERR_INVALID_STATUS;
        return -1;
    }

    pCoreHdl = (FAPEXCFDL_StrT*)FAPEXCFDL_GetCfdlHdl();

    if(pCoreHdl->fop[id]->release != NULL) {
        ret = pCoreHdl->fop[id]->release( id,
                                          (FAPEXFDLRELARG2_T)(pCoreHdl->pOsPrivate[id]) );
        if(ret < 0) {
            errno = -ret;
            return -1;
        }
        return ret;
    }
    return 0;
}

/*!
*******************************************************************************
**
** \brief  An ALI function (read method)
**         This function is called with the systemcall "read".
**
** \param  id     The device identifier.
** \param  pBuf   The pointer to the destination buffer which is in the user space.
** \param  size   The data size.
**
******************************************************************************/
FAPEXREADRET_T FAPEXSAL_Read(FAPEXREADARG1_T id, FAPEXREADARG2_T pBuf, FAPEXREADARG3_T size)
{
    FAPEXCFDL_StrT *pCoreHdl;
    FAPEXREADRET_T ret;

    if((id < 0) || (id >= FAPEX_MAXIMUM_MINOR)) {
        errno = -FAPEX_SYS_ERR_INVALID_STATUS;
        return -1;
    }

    pCoreHdl = (FAPEXCFDL_StrT*)FAPEXCFDL_GetCfdlHdl();

    if(pCoreHdl->fop[id]->read != NULL) {
        ret = pCoreHdl->fop[id]->read( (FAPEXFDLREADARG1_T)(pCoreHdl->pOsPrivate[id]),
                                       pBuf,
                                       size,
                                       0 /* not support */ );
        if(ret < 0) {
            errno = -ret;
            return -1;
        }
        return ret;
    }
    return 0;
}

/*!
*******************************************************************************
**
** \brief  An ALI function (write method)
**         This function is called with the systemcall "write".
**
** \param  id     The device identifier.
** \param  pBuf   The pointer to the source buffer which is in the user space.
** \param  size   The data size.
**
******************************************************************************/
FAPEXWRTRET_T FAPEXSAL_Write(FAPEXWRTARG1_T id, FAPEXWRTARG2_T pBuf, FAPEXWRTARG3_T size)
{
    FAPEXCFDL_StrT *pCoreHdl;
    FAPEXWRTRET_T  ret;

    if((id < 0) || (id >= FAPEX_MAXIMUM_MINOR)) {
        errno = -FAPEX_SYS_ERR_INVALID_STATUS;
        return -1;
    }

    pCoreHdl = (FAPEXCFDL_StrT*)FAPEXCFDL_GetCfdlHdl();

    if(pCoreHdl->fop[id]->write != NULL) {
        ret = pCoreHdl->fop[id]->write( (FAPEXFDLWRTARG1_T)(pCoreHdl->pOsPrivate[id]),
                                        pBuf,
                                        size,
                                        0 /* not support */ );
        if(ret < 0) {
            errno = -ret;
            return -1;
        }
        return ret;
    }
    return 0;
}

/*!
*******************************************************************************
**
** \brief  An ALI function (ioctl method)
**         This function is called with the systemcall "ioctl".
**
** \param  id     The device identifier.
** \param  cmd    The ioctl command.
** \param  pArg   The pointer to the argument of the ioctl command.
**
******************************************************************************/
FAPEXIOCRET_T FAPEXSAL_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg)
{
    FAPEXCFDL_StrT *pCoreHdl;
    FAPEXIOCRET_T  ret;

    if((id < 0) || (id >= FAPEX_MAXIMUM_MINOR)) {
        errno = -FAPEX_SYS_ERR_INVALID_STATUS;
        return -1;
    }

    pCoreHdl = (FAPEXCFDL_StrT*)FAPEXCFDL_GetCfdlHdl();

    if(pCoreHdl->fop[id]->ioctl != NULL) {
        ret = pCoreHdl->fop[id]->ioctl((FAPEXIOCARG1_T)id,
                                       (FAPEXFDLIOCARG2_T)(pCoreHdl->pOsPrivate[id]),
                                       (FAPEXFDLIOCARG3_T)cmd,
                                       (FAPEXFDLIOCARG4_T)pArg);
        if(ret < 0) {
            errno = -ret;
            return -1;
        }
        return ret;
    }
    return 0;
}

/*!
*******************************************************************************
**
** \brief  An ALI function (poll method)
**         This function is called with the systemcall "select".
**
** \param  id     The device identifier.
** \param  ptbl   Reserved (system internal). 
**                On Linux, may be, it is set NULL.
**                On RTOS, don't care, it is not used.
**
******************************************************************************/
FAPEXPLRET_T FAPEXSAL_Poll(FAPEXPLARG1_T id, FAPEXPLARG2_T ptbl)
{
    FAPEXCFDL_StrT   *pCoreHdl;
    FAPEXFDLPLRET_T  ret;

    if((id < 0) || (id >= FAPEX_MAXIMUM_MINOR)) {
        errno = -FAPEX_SYS_ERR_INVALID_STATUS;
        return -1;
    }

    pCoreHdl = (FAPEXCFDL_StrT*)FAPEXCFDL_GetCfdlHdl();

    if(pCoreHdl->fop[id]->poll != NULL) {
        ret = pCoreHdl->fop[id]->poll( (FAPEXFDLPLARG1_T)(pCoreHdl->pOsPrivate[id]),
                                       (FAPEXFDLPLARG2_T)ptbl);
        return ret;
    }
    return 0;
}

/*!
*******************************************************************************
**
** \brief  An ALI function (mmap method)
**         This function is called with the systemcall "mmap".
**
** \param  start  The start address
** \param  length The length of mapping area
** \param  prot   The length of mapping area
** \param  flags  The control flags
** \param  id     The device identifier.
** \param  offset The mapping area offset
**
******************************************************************************/
FAPEXMMPRET_T FAPEXSAL_Mmap(FAPEXMMPARG1_T start, FAPEXMMPARG2_T length,
                            FAPEXMMPARG3_T prot,  FAPEXMMPARG4_T flags,
                            FAPEXMMPARG5_T id,    FAPEXMMPARG6_T offset)
{
    FAPEXCFDL_StrT    *pCoreHdl;
    FAPEXFDLMMPRET_T  ret;
    FAPEXvm_area_S    vma;

    if((id < 0) || (id >= FAPEX_MAXIMUM_MINOR)) {
        errno = -FAPEX_SYS_ERR_INVALID_STATUS;
        return NULL;
    }

    pCoreHdl = (FAPEXCFDL_StrT*)FAPEXCFDL_GetCfdlHdl();

    if(pCoreHdl->fop[id]->mmap != NULL) {
        vma.vm_start  = (void *)start;
        vma.vm_length = length;
        vma.vm_offset = offset;
        ret = pCoreHdl->fop[id]->mmap( (FAPEXFDLMMPARG1_T)(pCoreHdl->pOsPrivate[id]),
                                       &vma);
        if(ret == 0) {
            return (FAPEXMMPRET_T)vma.vm_start;
        }
        errno = -ret;
    }
    else {
        errno = -FAPEX_SYS_ERR_FEATURE_NOT_SUPPORTED;
    }
    return NULL;
}

#endif /* !__LINUX__ */
