/*
 *
 * \attention  This program is provided as is. You can redistribute it and/or
 * modify it. Fujitsu Semiconductor Limited accepts no responsibility or
 * liability for any errors or omissions.
 *
 * Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
 */
/*
 FDL module skel_drv
*/

#include "fapex.h"
#include "fapex/abst_libir.h"
#include "fapex/sys_config.h"
#include "fapex_fdlir_priv.h"
#include "fapex_config_cmn.h"

/* for ir_funcStr */
#include "fapex_func_ext.h"

/*
*******************************************************************************
*******************************************************************************
** Global Data
*******************************************************************************
*******************************************************************************/

/*
*******************************************************************************
*******************************************************************************
** Local Data
*******************************************************************************
*******************************************************************************/
#ifndef ALLOCATE_DYNAMIC_MEMORY
static FAPEXFDL_IR_StrT  FAPIGLD_FDLIRCtx[FAPEXDEV_IR_DESCRIPTOR_NUM];
#endif /* !ALLOCATE_DYNAMIC_MEMORY */

/*!
*******************************************************************************
**
** \brief  Read method of this id's FDL.
**         This function is called with the systemcall "read".
**
** If copy data to the argument's buffer, use FAPI_SYS_copyFromUser().
**
** \param  file    The pointer to the struct file.
** \param  buf     The pointer to the base address of the buffer.
** \param  size    The buffer size.
** \param  offset  The offset (now, not supported).
**
******************************************************************************/
FAPEXFDLREADRET_T FAPEXFDLIR_read(FAPEXFDLREADARG1_T file, FAPEXFDLREADARG2_T buf,
                                  FAPEXFDLREADARG3_T size, FAPEXFDLREADARG4_T offset)
{
    FAPEXFDL_IR_StrT       *pfunc;
    FAPEXCFDLSAF_EventStrT *dt_event_ptr;
    int32_t   ret;
#ifdef __LINUX__
    uint32_t  *data;
#endif

    FAPEXDebugPrint(3, "[FAPEXFDLIR_read] size:%d\n",size);

    if((uint32_t)buf & 0x00000003) {
        return -1;    /* not 4bytes aligned */
    }

    /* Get the context base of this id's FDL. */
    pfunc = (FAPEXFDL_IR_StrT*)(file->private_data);

    dt_event_ptr = &pfunc->dt_event;
    if(pfunc->status == FAPEX_STATUS_RUNNING) {
        /* wait */
        dt_event_ptr->mode       = FAPEXCFDLSAF_WAITEVENT_SYNC | FAPEXCFDLSAF_WAITEVENT_OR;
        dt_event_ptr->wai_bitptn = FAPEXFDL_IR_EVENT_RECEIVE;
        dt_event_ptr->tout       = 0;

        ret = FAPEXCFDLSAF_WAIT_EVENT(dt_event_ptr);
    }
    else {
        /* clear read event */
        ret = FAPEXCFDLSAF_SET_EVENT(dt_event_ptr,
                                     ~FAPEXFDL_IR_EVENT_RECEIVE,
                                     FAPEXCFDLSAF_WAITEVENT_AND);
    }
    if(ret < 0) {
        if(ret == FAPI_ERR_TIMEOUT) {
            return FAPEX_IR_ERR_TIMEOUT;
        }
        return 0;
    }

#ifndef __LINUX__
    ret = ir_funcStr.ReadData(pfunc->pFAPICtxHdl, (uint32_t*)buf, size);

#else
    data = FAPI_SYS_MALLOC(size * sizeof(uint32_t));
    if(data == NULL) {
        return FAPEX_IR_ERR_OUT_OF_MEMORY;
    }

    ret = ir_funcStr.ReadData(pfunc->pFAPICtxHdl, data, size);

    if(ret == FAPI_OK) {
        /* copy parameters from the kernel space to an user space */
        ret = FAPEXCFDLSAF_COPY_TO_USR(buf, data, size);
    }
    FAPI_SYS_FREE(data);

#endif /* !__LINUX__ */

    if(ret == FAPI_OK) {
        return size;
    }
    FAPEXDebugPrint(2, "[FAPEXFDLIR_read] FAPI_IR_ReadData:error %d\n", ret);
    return -1;
}

#if 0 /* not supported */
/*!
*******************************************************************************
**
** \brief  Write method of this id's FDL.
**         This function is called with the systemcall "write".
**
** If copy data from the argument's buffer, use FAPI_SYS_copyToUser().
**
** \param  file    The pointer to the struct file.
** \param  buf     The pointer to the base address of the buffer.
** \param  size    The buffer size.
** \param  offset  The offset (now, not supported).
**
******************************************************************************/
FAPEXFDLWRTRET_T FAPEXFDLIR_write(FAPEXFDLWRTARG1_T file, FAPEXFDLWRTARG2_T buf,
                                  FAPEXFDLWRTARG3_T size, FAPEXFDLWRTARG4_T offset)
{
    FAPEXDebugPrint(3, "[FAPEXFDLIR_write] size:%d\n",size);

    return size;
}
#endif

/*!
*******************************************************************************
**
** \brief  IR Signal Notifer
**
** \param   handle       Valid driver instance handle previously returned by
**                       #FAPI_IR_Open.
**
******************************************************************************/
static void fdlir_signalNotifier(FAPI_SYS_HandleT handle, uint32_t* optDataPtr)
{
    FAPEXFDL_IR_StrT *pfunc = (FAPEXFDL_IR_StrT *)optDataPtr;

    /* set event */
    FAPEXCFDLSAF_SET_EVENT(
            &pfunc->dt_event,
            FAPEXFDL_IR_EVENT_RECEIVE,
            FAPEXCFDLSAF_WAITEVENT_OR);
}

/*!
*******************************************************************************
**
** \brief  IR descriptor open
**
** \param  pfunc_cmn    pointer to context
** \param  pArg         pointer to FAPEX_ICC_OpenParamStrT parameter
**
******************************************************************************/
static int32_t fdlir_Open(FAPEXFDL_IR_StrT *pfunc, FAPEX_IR_OpenParamStrT *pArg)
{
    FAPI_IR_OpenParamsT     params;
    FAPEXCFDLSAF_EventStrT  *dt_event_ptr;
    int32_t  ret;

    if(pfunc->pFAPICtxHdl != NULL) {
        return 0;
    }
    if(pArg == NULL) {
        return FAPEX_IR_ERR_BAD_PARAMETER;
    }
    /* copy parameters from an user space to the kernel space */
    FAPEXCFDLSAF_COPY_TO_DRV(&(pfunc->mode), &(pArg->mode), sizeof(uint32_t));

    /* setup open parameter */
    params.version = FAPI_IR_VERSION;
    switch(pfunc->mode) {
        case FAPEX_IR_MODE_RC5:
            params.swProtocol = ir_funcStr.RC5_Mode;
            break;

        case FAPEX_IR_MODE_NEC:
            params.swProtocol = ir_funcStr.NEC_Mode;
            break;

        default:
            params.swProtocol = NULL;
    }
    params.rcmNotifyFunction = fdlir_signalNotifier;
    params.optDataPtr        = (uint32_t*)pfunc;

    /* call IR open() */
    pfunc->pFAPICtxHdl = ir_funcStr.Open(&params, &ret);
    FAPEXDebugPrint(3, "[fdlir_Open] FAPI context:0x%08x\n",
                       (uint32_t)pfunc->pFAPICtxHdl);
    if(pfunc->pFAPICtxHdl != NULL) {
        pfunc->status = FAPEX_STATUS_RUNNING;

        /* setup event control */
        dt_event_ptr = &pfunc->dt_event;
        if(dt_event_ptr->initialized == 0) {
            FAPEXCFDLSAF_CREATE_EVENT(dt_event_ptr);
        }
        ret = 0;
    }
    else {
        FAPEXDebugPrint(1, "[fdlir_Open] FAPI_IR_Open error:%d\n", ret);
    }
    return ret;
}

/*!
*******************************************************************************
**
** \brief  General control of this id's FDL.
**         This function is called with systemcall "ioctl".
**
** \param  inode   The pointer to the struct inode.
** \param  file    The pointer to the struct file.
** \param  cmd     The ioctl command. This is set by user application. On Linux,
**                 the kernel copy this data to the kernel space. On RTOS, this
**                 memory area is shared with the user.
**                 memory of the user
** \param  arg     The argument of the ioctl command. This is set by user 
**                 application. On Linux, the kernel copy this data to the kernel
**                 space. On RTOS, thismemory area is shared with the user.
**
******************************************************************************/
FAPEXFDLIOCRET_T FAPEXFDLIR_ioctl(FAPEXFDLIOCARG1_T inode, FAPEXFDLIOCARG2_T file,
                                  FAPEXFDLIOCARG3_T cmd, FAPEXFDLIOCARG4_T arg)
{
    FAPEXFDL_IR_StrT *pfunc;
    int32_t  ret = FAPEX_IR_ERR_BAD_PARAMETER;

    FAPEXDebugPrint(3, "[FAPEXFDLIR_ioctl] cmd:%08x\n",cmd);

    /* Get the context base of this id's FDL. */
    pfunc = (FAPEXFDL_IR_StrT*)(file->private_data);

    switch(cmd) {
        /* actual code which is depend on function is implimented here. */
        case FAPEX_IOCCOM_IR_GET_KEY:
            if(arg != (FAPEXFDLIOCARG4_T)NULL) {
                FAPEX_IR_KeyParamStrT  params;
                FAPEXCFDLSAF_EventStrT *dt_event_ptr;

                dt_event_ptr = &pfunc->dt_event;
                if(pfunc->status == FAPEX_STATUS_RUNNING) {
                    /* wait */
                    dt_event_ptr->mode       = FAPEXCFDLSAF_WAITEVENT_SYNC | FAPEXCFDLSAF_WAITEVENT_OR;
                    dt_event_ptr->wai_bitptn = FAPEXFDL_IR_EVENT_RECEIVE;
                    dt_event_ptr->tout       = 0;

                    ret = FAPEXCFDLSAF_WAIT_EVENT(dt_event_ptr);
                }
                else {
                    /* clear read event */
                    ret = FAPEXCFDLSAF_SET_EVENT(dt_event_ptr,
                                                 ~FAPEXFDL_IR_EVENT_RECEIVE,
                                                 FAPEXCFDLSAF_WAITEVENT_AND);
                }

                if(ret != FAPI_ERR_TIMEOUT) {
                    /* get key data */
                    ret = ir_funcStr.GetKey(pfunc->pFAPICtxHdl,
                                            &params.deviceId,
                                            &params.keyValue,
                                            &params.rcmCode,
                                            &params.repeatFlag);
                    if(ret == FAPI_OK) {
                        /* copy parameters from the kernel space to an user space */
                        FAPEXCFDLSAF_COPY_TO_USR((FAPEX_IR_KeyParamStrT*)(arg), &params,
                                                 sizeof(FAPEX_IR_KeyParamStrT));
                    }
                    else {
                        FAPEXDebugPrint(2, "[FAPEXFDLIR_ioctl] "
                                           "FAPI_IR_GetKey:error %d\n", ret);
                    }
                }
                else {
                    ret = FAPEX_IR_ERR_TIMEOUT;
                }
            }
            break;

        case FAPEX_IOCCOM_IR_OPEN:
            if(pfunc != NULL) {
                ret = fdlir_Open(pfunc, (FAPEX_IR_OpenParamStrT *)arg);
            }
            break;

        case FAPEX_IOCCOM_IR_SET_MAPTABLE:
            if(arg != (FAPEXFDLIOCARG4_T)NULL) {
                FAPI_IR_MappingTableT       params;
                FAPI_IR_KeycodeMapT         *map_dst;
                FAPEX_IR_KeycodeMapStrT     *map_src;
                FAPEX_IR_MapTableParamStrT  *pArg = (FAPEX_IR_MapTableParamStrT *)arg;
                uint32_t                    i, max_num;

                /* copy parameters from an user space to the kernel space */
                FAPEXCFDLSAF_COPY_TO_DRV(&(params.mDeviceId), &(pArg->deviceId),   sizeof(int32_t));
                FAPEXCFDLSAF_COPY_TO_DRV(&max_num,            &(pArg->keycodeNum), sizeof(uint32_t));

                if(pfunc->keycode_table != NULL) {
                    if(pfunc->keycode_num < max_num) {
                        FAPI_SYS_FREE(pfunc->keycode_table);
                        pfunc->keycode_table = NULL;
                    }
                }
                if(pfunc->keycode_table == NULL) {
                    pfunc->keycode_table = (FAPI_IR_KeycodeMapT *)FAPI_SYS_MALLOC(
                                                                (max_num * sizeof(FAPI_IR_KeycodeMapT)));
                }
                if(pfunc->keycode_table != NULL) {
                    pfunc->keycode_num = max_num;

                    /* copy parameters from an user space to the kernel space */
                    map_src = pArg->pKeycodeTable;
                    map_dst = pfunc->keycode_table;
                    for(i=0; i<max_num; i++) {
                        map_dst->version = FAPI_IR_VERSION;
                        /* copy parameters from an user space to the kernel space */
                        FAPEXCFDLSAF_COPY_TO_DRV(&(map_dst->keyValue), &(map_src->keyValue),
                                                 sizeof(uint32_t));
                        FAPEXCFDLSAF_COPY_TO_DRV(&(map_dst->rcmCode),  &(map_src->rcmCode),
                                                 sizeof(uint32_t));
                        map_src ++;
                        map_dst ++;
                    }

                    /* setup parameter */
                    params.version   = FAPI_IR_VERSION;
                    params.mTablePtr = pfunc->keycode_table;

                    ret = ir_funcStr.SetMapTable(&params);
                    if(ret != FAPI_OK) {
                        FAPEXDebugPrint(2, "[FAPEXFDLIR_ioctl] "
                                           "FAPI_IR_SetMapTable:error %d\n", ret);
                    }
                }
                else {
                    ret = FAPEX_IR_ERR_OUT_OF_MEMORY;
                }
            }
            break;

        case FAPEX_IOCCOM_IR_REMOVE_MAPTABLE:
            {
                int32_t  device_id = (int32_t)arg;

                ret = ir_funcStr.RemoveMapTable(device_id);
                if(ret != FAPI_OK) {
                    FAPEXDebugPrint(2, "[FAPEXFDLIR_ioctl] "
                                       "FAPI_IR_RemoveMapTable:error %d\n", ret);
                }
            }
            break;

        case FAPEX_IOCCOM_IR_GET_STATUS:
            if(arg != (FAPEXFDLIOCARG4_T)NULL) {
                FAPEX_IR_StatusStrT params;

                /* call IR open() */
                params.status = ir_funcStr.GetStatus(pfunc->pFAPICtxHdl);

                /* copy parameters from the kernel space to an user space */
                FAPEXCFDLSAF_COPY_TO_USR((FAPEX_IR_StatusStrT*)(arg), &params, sizeof(FAPEX_IR_StatusStrT));
                ret = 0;
            }
            break;

        default:
            ;
    }

    return ret;
}

/*!
*******************************************************************************
**
** \brief  Poll method of this id's FDL.
**         This function is called with the systemcall "select".
**
** \param  file    The pointer to the struct file.
** \param  ptbl    An internal data.
**
******************************************************************************/
FAPEXFDLPLRET_T FAPEXFDLIR_poll(FAPEXFDLPLARG1_T file, FAPEXFDLPLARG2_T ptbl)
{
    FAPEXFDLPLRET_T        mask = 0;
    FAPEXFDL_IR_StrT       *pfunc;
    FAPEXCFDLSAF_EventStrT *dt_event_ptr;

    /* Get the context base of this id's FDL. */
    pfunc = (FAPEXFDL_IR_StrT*)(file->private_data);
    dt_event_ptr = &pfunc->dt_event;

    FAPEXDebugPrint(4, "[FAPEXFDLIR_poll]\n");

    FAPEXCFDLSAF_POLL_EVENT(file, dt_event_ptr, NULL, ptbl);

    /* actual code which is depend on function is implimented here. */

    if(dt_event_ptr->evt_bitptn & FAPEXFDL_IR_EVENT_RECEIVE) {
        mask |= POLLIN | POLLRDNORM;
    }

    return mask;
}

#if 0 /* not supported */
/*!
*******************************************************************************
**
** \brief  Mmap method of this id's FDL.
**         This function is called with the systemcall "mmap".
**
** \param  file    The pointer to the struct file.
** \param  vma     system depend
**
******************************************************************************/
FAPEXFDLMMPRET_T FAPEXFDLIR_mmap(FAPEXFDLMMPARG1_T file, FAPEXFDLMMPARG2_T vma)
{
#ifdef __LINUX__
    int32_t ret;

    if (vma->vm_pgoff != 0){
        return -EINVAL;
    }

    /* This area doen't swapout */
    vma->vm_flags |= VM_IO | VM_RESERVED;

    /* This area is IO area. Set noncache, and it doesn't dump to a core
       file of a process */
    if (file->f_flags & O_SYNC){
        vma->vm_page_prot = pgprot_noncached(vma->vm_page_prot);
        vma->vm_flags |= VM_IO;
    }

    ret = remap_pfn_range(vma, vma->vm_start, vma->vm_pgoff,
                  vma->vm_end - vma->vm_start, vma->vm_page_prot);

    if(ret != 0){
        return -EAGAIN;
    }
    return 0;
#else
    return -1;
#endif /* __LINUX__ */
}
#endif

/*!
*******************************************************************************
**
** \brief  Open this id's FDL.
**         This function is called with the systemcall "open". And this id's FDL 
**         are opened.
**
** \param  inode   The pointer to the struct inode.
** \param  file    The pointer to the struct file.
**
******************************************************************************/
FAPEXFDLOPENRET_T FAPEXFDLIR_open (FAPEXFDLOPENARG1_T inode, FAPEXFDLOPENARG2_T file)
{
    FAPEXFDL_IR_StrT *pfunc;
    void    *pCfdlHdl;
    int32_t id;

    FAPEXDebugPrint(3, "[FAPEXFDLIR_open] file:0x%08x,private_data:0x%08x\n",
                       (uint32_t)file, (uint32_t)file->private_data);

    if(file->private_data == NULL){
        id = FAPEXCFDLSAF_GETID(inode);
        FAPEXDebugPrint(3, "[FAPEXFDLIR_open] minor_no:%d\n", id);

        /* Get the context base of the CFDL. */
        pCfdlHdl = (void*)FAPEXCFDL_GetCfdlHdl();
        FAPEXDebugPrint(3, "[FAPEXFDLIR_open] pCfdlHdl:0x%08x\n", (uint32_t)(pCfdlHdl));

        /* Get the context base of this id's FDL. */
        pfunc = (FAPEXFDL_IR_StrT*)FAPEXCFDL_GetFdlHdl(pCfdlHdl, id);
        file->private_data = pfunc;
        FAPEXDebugPrint(3, "[FAPEXFDLIR_open] pFdlHdl:0x%08x\n", (uint32_t)(pfunc));

        /* open sequence */
#ifdef __LINUX__
        kref_init(&pfunc->kref);
#endif
    }
    else {
        pfunc = (FAPEXFDL_IR_StrT *)file->private_data;
#ifdef __LINUX__
        kref_get(&pfunc->kref);
#endif
    }

    /* increment our usage count for the device */
    pfunc->used_count++;

    FAPEXDebugPrint(3, "[FAPEXFDLIR_open] fapex_fdlir was opened : used_counter %d.\n",
                       pfunc->used_count);

    return 0;
}

/*!
*******************************************************************************
**
** \brief  delete this FDL id's module.
**         This function is called with the systemcall "close". And this id's FDL
**         are released.
**
** \param  kref    The pointer to the kref structure.
**
******************************************************************************/
#ifdef __LINUX__
static void fdlir_delete(struct kref *kref)
{
    FAPEXFDL_IR_StrT  *pfunc;
    int32_t  ret = 0;

    pfunc = container_of(kref, FAPEXFDL_IR_StrT, kref);
#else
static int32_t fdlir_delete(FAPEXFDL_IR_StrT *pfunc)
{
    int32_t  ret = 0;
#endif

    if(pfunc->pFAPICtxHdl != NULL) {
        pfunc->status = FAPEX_STATUS_OPENED;

        /* close sequence */
        if(pfunc->keycode_table != NULL) {
            FAPI_SYS_FREE(pfunc->keycode_table);
        }
        /* clear event control */
        FAPEXCFDLSAF_DELETE_EVENT(&pfunc->dt_event);

#ifdef FAPI_IR_CONNECT
        /* close sequence */
        ret = ir_funcStr.Close(pfunc->pFAPICtxHdl);
        if(ret != FAPI_OK) {
            FAPEXDebugPrint(2, "[fdlir_delete] "
                               "FAPI_IR_Close:error %d\n", ret);
            goto fdlir_delete_end;
        }
#endif /* FAPI_IR_CONNECT */
        pfunc->pFAPICtxHdl = NULL;
    }
    pfunc->status = FAPEX_STATUS_TERMINATED;

fdlir_delete_end:
#ifdef __LINUX__
    pfunc->close_result = ret;
#else
    return ret;
#endif
}

/*!
*******************************************************************************
**
** \brief  Release this FDL id's module.
**         This function is called with the systemcall "close". And this id's FDL
**         are released.
**
** \param  inode   The pointer to the struct inode.
** \param  file    The pointer to the struct file.
**
******************************************************************************/
FAPEXFDLRELRET_T FAPEXFDLIR_release (FAPEXFDLRELARG1_T inode, FAPEXFDLRELARG2_T file)
{
    FAPEXFDL_IR_StrT  *pfunc;
    FAPEXFDLRELRET_T  ret = 0;

    /* Get the context base of this id's FDL. */
    pfunc = (FAPEXFDL_IR_StrT*)(file->private_data);

    if(pfunc != NULL) {
        /* decrement the count on our device */
        pfunc->used_count--;
#ifdef __LINUX__
        ret = kref_put(&pfunc->kref, fdlir_delete);
        if(ret != 0) {
            ret = pfunc->close_result;
        }
#else
        ret = fdlir_delete(pfunc);
#endif

        FAPEXDebugPrint(3, "[FAPEXFDLIR_release] : used_count %d\n",
                           pfunc->used_count);
    }
    FAPEXFDL_RETVAL(ret);
}

/*--------------------------------------------------------*/
/* The file operations of this FDL module.                */
/*--------------------------------------------------------*/
struct file_operations fapex_fdlir_fops;

/*!
*******************************************************************************
**
** \brief  Initialize process of the FDL module.
**         This function is called when the FDL module is loaded. It is executed
**         only one times.
**         On Linux, this function is registered to the kernel using "module_init".
**         And user application doesn't call this function. If do "insmod" command,
**         this is called by the kernel.
**         On RTOS, user application has to call this function.
**
** \param  void (can't set any argument)
**
******************************************************************************/
int32_t FAPEXFDLIR_init(void)
{
    int32_t ret;
    int32_t count;
    FAPEXFDL_IR_StrT    *pfunc;
    FAPEXCFDL_UtilStrT  utility;
#ifdef ALLOCATE_DYNAMIC_MEMORY
    FAPEXFDL_IR_StrT    *pfunc0;
#endif

    FAPEXDebugPrint(1, "[FAPEXFDLIR_init]\n");

    /* set struct file_operations */
    fapex_fdlir_fops.owner   = THIS_MODULE;
    fapex_fdlir_fops.open    = FAPEXFDLIR_open;
    fapex_fdlir_fops.read    = FAPEXFDLIR_read;
//  fapex_fdlir_fops.write   = FAPEXFDLIR_write;
    fapex_fdlir_fops.ioctl   = FAPEXFDLIR_ioctl;
    fapex_fdlir_fops.poll    = FAPEXFDLIR_poll;
//  fapex_fdlir_fops.mmap    = FAPEXFDLIR_mmap;
    fapex_fdlir_fops.release = FAPEXFDLIR_release;

#ifdef ALLOCATE_DYNAMIC_MEMORY
    pfunc = (FAPEXFDL_IR_StrT*)FAPI_SYS_MALLOC(sizeof(FAPEXFDL_IR_StrT)*(FAPEXDEV_IR_DESCRIPTOR_NUM));
    if(pfunc == NULL) {
        return FAPEX_IR_ERR_OUT_OF_MEMORY;
    }
    pfunc0 = pfunc;
#else
    pfunc = FAPIGLD_FDLIRCtx;
#endif /* ALLOCATE_DYNAMIC_MEMORY */
    FAPEXCFDLSAF_CLEAR_MEMORY(pfunc, sizeof(FAPEXFDL_IR_StrT)*(FAPEXDEV_IR_DESCRIPTOR_NUM));

    for(count=FAPEXDEV_IR_MINOR_FIRST; count<=FAPEXDEV_IR_MINOR_LAST; count++){
        /* Register this id's FDL device to CFDL. */
        ret = FAPEXCFDL_RegisterFdl(&fapex_fdlir_fops, pfunc, &utility, count);
        if(ret) {
#ifdef ALLOCATE_DYNAMIC_MEMORY
            FAPI_SYS_FREE(pfunc0);
#endif
            FAPEXDebugPrint(1, "[FAPEXFDLIR_init] FAPEXCFDL_RegisterFdl:error %d\n", ret);
            return ret;
        }
        pfunc ++;
    }

#if defined(FAPI_IR_CONNECT) && !defined(FUJITSU_MB86H60_INPUT_IRDEV)
    FAPEXDebugPrint(1, "call FAPEXCFG_IR_Init\n");

    /* call FAPI (initialize) */
    ret = FAPEXCFG_IR_Init();
#else
    ret = 0;
#endif /* FAPI_IR_CONNECT && !FUJITSU_MB86H60_INPUT_IRDEV */

    FAPEXDebugPrint(1, "[FAPEXFDLIR_init] fapex_fdlir was loaded. %d\n", ret);

    return ret;
}

/*!
*******************************************************************************
**
** \brief  Cleanup process of the FDL module.
**         This function is called when the FDL module is unloaded. It is executed
**         only one times.
**         On Linux, this function is registered to the kernel using "module_exit".
**         And user application doesn't call this function. If do "rmmod" command, 
**         this is called by the kernel.
**         On RTOS, user application has to call this function.
**
** \param  void (can't set any argument)
**
******************************************************************************/
void FAPEXFDLIR_cleanup(void)
{
    int32_t count;

    FAPEXDebugPrint(1, "[FAPEXFDLIR_cleanup]\n");

#if defined(FAPI_IR_CONNECT) && !defined(FUJITSU_MB86H60_INPUT_IRDEV)
    /* call FAPI API (cleanup) */
    FAPEXCFG_IR_Cleanup();

    FAPEXDebugPrint(1, "end FAPEXCFG_IR_Cleanup\n");
#endif /* FAPI_IR_CONNECT && !FUJITSU_MB86H60_INPUT_IRDEV */

    /* cleanup sequence (common this FDL) */
#ifdef ALLOCATE_DYNAMIC_MEMORY
    {
        void             *pCfdlHdl;
        FAPEXFDL_IR_StrT *pfunc;

        pCfdlHdl = (void*)FAPEXCFDL_GetCfdlHdl();
        pfunc    = (FAPEXFDL_IR_StrT*)FAPEXCFDL_GetFdlHdl(pCfdlHdl, FAPEXDEV_IR_MINOR_FIRST);

        /* relase the FDL context */
        if(pfunc != NULL) {
            FAPI_SYS_FREE(pfunc);
        }
    }
#endif /* ALLOCATE_DYNAMIC_MEMORY */

    for(count=FAPEXDEV_IR_MINOR_FIRST; count<=FAPEXDEV_IR_MINOR_LAST; count++) {
        /* Unregister this id's FDL device to CFDL. */
        FAPEXCFDL_UnregisterFdl(count);
    }

    FAPEXDebugPrint(1, "[FAPEXFDLIR_cleanup] fapex_fdlir was unloaded.\n");
    return;
}

#ifdef __LINUX__
/*******************************
 MACRO (Linux dep.)
********************************/
#ifdef BUILD_AS_MODULE
module_init(FAPEXFDLIR_init);
module_exit(FAPEXFDLIR_cleanup);

MODULE_LICENSE("Proprietary. Fujitsu Semiconductor.");
MODULE_DESCRIPTION("FAPI driver (fapex_fdlir object)");
#endif /* BUILD_AS_MODULE */
#endif /* __LINUX__ */
