/*!
*****************************************************************************
**
** \file        fapex/abst_libaudec_ac3.h
** 
** \version     $Id: drv_audec_ac3.h 4543 2009-07-31 13:59:13Z spurke $
**
** \brief       AC3 codec specific parameters.
**
** This file contains the declaration of specific parameters for the AC3 codec.
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** (C) Copyright 2006-2010 Fujitsu Semiconductor Europe GmbH, All Rights Reserved.
** (C) Copyright 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*****************************************************************************
*/

#ifndef _FAPEX_ABST_LIBAUDEC_AC3_H
    #define _FAPEX_ABST_LIBAUDEC_AC3_H

    #include <fapex/abst_lib.h>

    //***********************************************************************
    //***********************************************************************
    //** Defines and Macros
    //***********************************************************************
    //***********************************************************************


    //***********************************************************************
    //***********************************************************************
    //** Data types
    //***********************************************************************
    //***********************************************************************


    /*!
    *************************************************************************
    ** \brief Parameters for the AC3 decoder.
    **
    ** This data structure covers all parameters which can be set for the AC3
    ** decoder.
    ** Setting these parameters is optional. If not the codec applies the 
    ** mentioned defaults. 
    **
    ** \note The various parameters are defined by the Dolby Sotware Interface
    **       Protocol.  
    **
    ** \sa FAPEX_AUDEC_Ioctl/FAPEX_IOCCOM_AUDEC_SET_CODEC_PARAMS
    *************************************************************************
    */
    typedef struct {

        /*!
           The audio decoder ID.
           \note This field must be set to FAPEX_AUDEC_CODEC_TYPE_AC3.
        */
        int32_t  decoder;
        
        /*!
           Dolby SIP compression mode.
           A value between 0 and 3. (Codec's default: 2)
        */
        uint32_t compMode;
        
        /*!
           Dolby SIP stereo output mode.
           A value between 0 and 2. (Codec's default: 0)
        */
        uint32_t stereoMode;
        
        /*!
           Dolby SIP dual mono reproduction mode.
           A value between 0 and 3. (Codec's default: 0)
        */
        uint32_t dualMonoMode;
        
        /*!
           Dolby SIP output channel configuration.
           A value between 0 and 15. (Codec's default: -1)
           
           \note Bit 3 determines the LFE presence and bit 0-2
           corresponds to the channel configuration code as described
           by the Dolby SIP document.
           
           The output channel configuration is usually automatically derived 
           from the AUOUT driver's speaker configuration. Sometimes 
           it could be desirable to overwrite this approach. For this purpose 
           this parameter can be used. If the codec's current channel 
           configuration should not be overwritten it must be set to -1.
        */
        int32_t outChannelCfg;
        
        /*!
           Dolby SIP maximum repeat count before muting. 
           A value between 0 and 2**15. (Codec's default: 0)
        */
        uint32_t maxRepCount;
        
        /*!
           Enable AC-3 through SPDIF. 
           This switch tells the AC3 decoder to output the  
           AC3 bitstream also via SPDIF. Otherwise it will output PCM of the first 
           two speakers in the associated audio output (AUOUT) via SPDIF as usual.
           \note This switch takes only effect if SPDIF has been enabled for the
                 audio output associated with the AC3 decoder.
           A value of 0 or 1. (Codec's default: 0)
        */
        uint32_t ac3ToSpdif;
        
        /*! dummy to achieve compatible alignment between ARM and ARC */
        uint32_t dummy;
        
        /*!
           Dolby SIP dynamic range scale factor high level signals.
           A value between 0.0 and 1.0. (Codec's default: 1.0)
        */        
        double dynsclh;
        
        /*!
           Dolby SIP dynamic range scale factor low level signals.
           A value between 0.0 and 1.0. (Codec's default: 1.0)
        */  
        double dynscll;
        
        /*!
           Dolby SIP PCM scale factor.
           A value between 0.0 and 1.0. (Codec's default: 1.0)
        */  
        double pcmscl;
    } FAPEX_AUDEC_Ac3ParamStrT;

#endif
