/*!
*************************************************************************
**
** \file    fapex/abst_libboardifc.h
**
** \brief   abstraction library header file (BOARDIFC driver)
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#ifndef _FAPEX_ABST_LIBBOARDIFC_H
#define _FAPEX_ABST_LIBBOARDIFC_H

#include "fapex/abst_lib.h"
#include "fapex/abst_error.h"

/*
*************************************************************************
*************************************************************************
** Defines and Macros
*************************************************************************
*************************************************************************/

/*
*************************************************************************
** Standard error codes for the driver
*************************************************************************/
/*! FAPex BOARDIFC error code base value */
#define FAPEX_BOARDIFC_ERR_BASE             (FAPEX_BOARDIFC_MODULE_BASE + FAPEX_RET_ERR_BASE)

/*! Bad parameter passed. */
#define FAPEX_BOARDIFC_ERR_BAD_PARAMETER    (FAPEX_BOARDIFC_ERR_BASE + FAPEX_ERR_BAD_PARAMETER)

/*! Memory allocation failed. */
#define FAPEX_BOARDIFC_ERR_OUT_OF_MEMORY    (FAPEX_BOARDIFC_ERR_BASE + FAPEX_ERR_OUT_OF_MEMORY)

/*
*************************************************************************
*************************************************************************
** Data types
*************************************************************************
*************************************************************************/
#define FAPEX_BOARD_HWCONFIG_VERSION  0xFABE0003UL  //!< magic to validate 'FAPI_BOARD_HwPlatformInfosT'
#define FAPEX_BOARD_MAX_TSINP_PATH  0x02            //!< max. supported (effective) TS-Input.

#define FAPEX_BOARD_RSRC_TSIN_NONE     0x00         //!< TS input:none.
#define FAPEX_BOARD_RSRC_TSIN_FE1      0x01         //!< TS input:Frontend 1.
#define FAPEX_BOARD_RSRC_TSIN_FE2      0x02         //!< TS input:Frontend 2.
#define FAPEX_BOARD_RSRC_TSIN_MASK     0x03         //!< TS valid input bits
#define FAPEX_BOARD_RSRC_TSIN_SERIAL   0x10         //!< bitFlag: TS-Input is 'serial-mode'

#define FAPEX_BOARD_RSRC_CIIN_NONE     0x00         //!< CI input:none.
#define FAPEX_BOARD_RSRC_CIIN_FE1      0x01         //!< CI input:Frontend 1.
#define FAPEX_BOARD_RSRC_CIIN_FE2      0x02         //!< CI input:Frontend 2.
#define FAPEX_BOARD_RSRC_CIIN_TSGEN    0x04         //!< CI input:other TS(non-frontend).

#define FAPEX_BOARD_BITFLAG_PFLASH     0x01         //!<P-Flash available on HW.
#define FAPEX_BOARD_BITFLAG_SFLASH     0x02         //!<S-Flash available on HW.
#define FAPEX_BOARD_BITFLAG_NFLASH     0x04         //!<N-Flash available on HW.  

typedef struct
{
    uint8_t   defaultFeType;        //!< see 'FE_SYSTEM_TYPE_SATELLITE' etc.
    uint8_t   tsPathInputConfig;    //!< TS input (one of FAPI_BOARD_RSRC_TSIN_XXX).
    uint8_t   flagCiPathAval;       //!< '0' or '1'
    uint8_t   inputOptionsCiPath;   //!< CI input options (or'ed
                                    //!< bitfield of FAPI_BOARD_RSRC_CIIN_XXX).
} FAPEX_BOARD_FeConfigInfoStrT;

#define FAPEX_BOARD_HD1_P_ATA   0x01    //!<'P-ATA' HardDisk[1]-Interface available
#define FAPEX_BOARD_HD2_P_ATA   0x02    //!<'P-ATA' HardDisk[2]-Interface available
#define FAPEX_BOARD_HD1_S_ATA   0x10    //!<'S-ATA' HardDisk[1]-Interface available(via S-Ata-Adapter)
#define FAPEX_BOARD_HD2_S_ATA   0x20    //!<'S-ATA' HardDisk[2]-Interface available

#define FAPEX_BOARD_VID_AMP_C0  0x00    //!<video/scaler coefficients optimized like for 'FME-DK01'
#define FAPEX_BOARD_VID_AMP_C1  0x01    //!<video/scaler coefficients optimized like ofr 'FMC-PDK'

/*!
*******************************************************************************
** \brief HW-platform/board 'Info-Structure'
**
** The related ioctl command type : FAPEX_IOCCOM_BOARDIFC_GETBOARDINFO
**
** \sa FAPEX_BOARDIFC_Ioctl
******************************************************************************
*/
typedef struct
{
    uint32_t                 MagicAndVersion;        //!< BOARD_HWCONFIG_VERSION
    const char_t            *pHwManufactString;      //!< Manufacturer-Info
    const char_t            *pHwModelString;         //!< HW-Model Info
    const void              *pOptAdditionalInfo;     //!< e.g. can be used for spec. test purpose
    uint8_t                  flagUsbHostOnly;        //!<
    uint8_t                  typeOfFlashAvailable;   //!< see 'FAPI_BOARD_BITFLAG_PFLASH' etc.
    uint8_t                  AudioConfigurationInfoA;//!< for future use
    uint8_t                  AudioConfigurationInfoB;//!< for future use
    uint8_t                  numberOfAvlFrontEnds;   //!< number of available frontends
    uint8_t                  numberOfAvlCiSlots;     //!< number of available CI-Slots
    FAPEX_BOARD_FeConfigInfoStrT feCfg[FAPEX_BOARD_MAX_TSINP_PATH];  //!< FE-CfgData
    uint8_t                  flagHardDiskInterface;  //!< see 'FAPI_BOARD_HD1_P_ATA' etc.
    uint8_t                  flgDdr2MemoryInfos;     //!< not really used yet
    uint8_t                  videoAmpCharacteristic; //!< see 'FAPI_BOARD_VID_AMP_C0' etc.

} FAPEX_BOARD_HwPlatformInfosStrT;

/*!
*************************************************************************
** \brief ioctl specific command for the BOARDIFC driver
*************************************************************************/
#define FAPEX_IOCCOM_BOARDIFC_INIT_DETECT_FUNCTION              _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_BOARDIFC + 0, uint32_t)

/* not support in this release */
/*
#define FAPEX_IOCCOM_BOARDIFC_CONTROL_TEST_LED_KEY              _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_BOARDIFC + 1, uint32_t)
#define FAPEX_IOCCOM_BOARDIFC_CONTROL_FE                        _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_BOARDIFC + 2, uint32_t)
#define FAPEX_IOCCOM_BOARDIFC_CONTROL_FE_DETECT                 _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_BOARDIFC + 3, uint32_t)
#define FAPEX_IOCCOM_BOARDIFC_CONTROL_HD                        _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_BOARDIFC + 4, uint32_t)
#define FAPEX_IOCCOM_BOARDIFC_CONTROL_SCART                     _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_BOARDIFC + 5, uint32_t)
#define FAPEX_IOCCOM_BOARDIFC_EXT_CONTROL_TS_PATH               _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_BOARDIFC + 6, int32_t)
#define FAPEX_IOCCOM_BOARDIFC_EXT_CONTROL_HDMI_PATH             _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_BOARDIFC + 7, int32_t)
#define FAPEX_IOCCOM_BOARDIFC_DIAGNOSTIC_TS_PATH                _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_BOARDIFC + 8, uint32_t)
#define FAPEX_IOCCOM_BOARDIFC_DIAGNOSTIC_HDMI_PATH              _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_BOARDIFC + 9, uint32_t)
#define FAPEX_IOCCOM_BOARDIFC_DIAGNOSTIC_CI_CONTROL             _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_BOARDIFC + 10, uint32_t)
#define FAPEX_IOCCOM_BOARDIFC_DIAGNOSTIC_GENERALPURPOSE         _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_BOARDIFC + 11, uint32_t)
*/
#define FAPEX_IOCCOM_PLATFORM_SUPPORT                           _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_BOARDIFC + 12, uint32_t)
#define FAPEX_IOCCOM_FRONTEND_SUPPORT                           _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_BOARDIFC + 13, uint32_t)
#define FAPEX_IOCCOM_PLATFORM_SUPPORT_EXT                       _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_BOARDIFC + 14, uint32_t)
#define FAPEX_IOCCOM_BOARDIFC_OPEN                              _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_BOARDIFC + 15, uint32_t)
#define FAPEX_IOCCOM_BOARDIFC_GETBOARDINFO                      _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_BOARDIFC + 0, void*)

/*
*************************************************************************
*************************************************************************
** prototype for application
*************************************************************************
*************************************************************************/
#ifndef __LINUX__
#include "fapex/abst_types.h"

#ifdef __cplusplus
}
#endif

extern int32_t fd_bd;
extern FAPEX_BOARD_HwPlatformInfosStrT hwInfo;

FAPEXOPENRET_T FAPEX_BOARDIFC_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr);
FAPEXRELRET_T  FAPEX_BOARDIFC_Close(FAPEXRELARG1_T id);
FAPEXIOCRET_T  FAPEX_BOARDIFC_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg);

#define FAPI_BOARD_INIT_DETECT_FUNCTION( optInitBitFlags , optPtr)                  \
        if(fd_bd<0)                                                                 \
        {                                                                           \
            fd_bd = FAPEX_BOARDIFC_Open(FAPEXDEV_BOARDIFC_MINOR_FIRST,O_RDWR,NULL); \
        }                                                                           \
        if(fd_bd >= 0)                                                              \
        {                                                                           \
            FAPEX_BOARDIFC_Ioctl( fd_bd, FAPEX_IOCCOM_BOARDIFC_INIT_DETECT_FUNCTION, (FAPEXIOCARG3_T)NULL );    \
        }

#define FAPI_BOARD_CONTROL_SCART( index, command, optPtr )                          \
        if(fd_bd<0)                                                                 \
        {                                                                           \
            fd_bd = FAPEX_BOARDIFC_Open(FAPEXDEV_BOARDIFC_MINOR_FIRST,O_RDWR,NULL); \
        }                                                                           \
        if(fd_bd >= 0)                                                              \
        {                                                                           \
            FAPEX_BOARDIFC_Ioctl( fd_bd, FAPEX_IOCCOM_BOARDIFC_CONTROL_SCART, (FAPEXIOCARG3_T)NULL );    \
        }

#define FAPI_BOARD_GETBOARDINFO(  )                                                 \
        if(fd_bd<0)                                                                 \
        {                                                                           \
            fd_bd = FAPEX_BOARDIFC_Open(FAPEXDEV_BOARDIFC_MINOR_FIRST,O_RDWR,NULL); \
        }                                                                           \
        if(fd_bd >= 0)                                                              \
        {                                                                           \
            FAPEX_BOARDIFC_Ioctl( fd_bd, FAPEX_IOCCOM_BOARDIFC_GETBOARDINFO, (FAPEXIOCARG3_T)&hwInfo );    \
        }

#define BOARDS_INSTALL_PLATFORM_SUPPORT(  )                                         \
        if(fd_bd<0)                                                                 \
        {                                                                           \
            fd_bd = FAPEX_BOARDIFC_Open(FAPEXDEV_BOARDIFC_MINOR_FIRST,O_RDWR,NULL); \
        }                                                                           \
        if(fd_bd >= 0)                                                              \
        {                                                                           \
            FAPEX_BOARDIFC_Ioctl( fd_bd, FAPEX_IOCCOM_PLATFORM_SUPPORT, (FAPEXIOCARG3_T)NULL );    \
        }

#define BOARDS_INSTALL_FRONTEND_SUPPORT(  )                                         \
        if(fd_bd<0)                                                                 \
        {                                                                           \
            fd_bd = FAPEX_BOARDIFC_Open(FAPEXDEV_BOARDIFC_MINOR_FIRST,O_RDWR,NULL); \
        }                                                                           \
        if(fd_bd >= 0)                                                              \
        {                                                                           \
            FAPEX_BOARDIFC_Ioctl( fd_bd, FAPEX_IOCCOM_FRONTEND_SUPPORT, (FAPEXIOCARG3_T)NULL );    \
        }

#define BOARDS_INSTALL_PLATFORM_SUPPORT_EXT(  )                                     \
        if(fd_bd<0)                                                                 \
        {                                                                           \
            fd_bd = FAPEX_BOARDIFC_Open(FAPEXDEV_BOARDIFC_MINOR_FIRST,O_RDWR,NULL); \
        }                                                                           \
        if(fd_bd >= 0)                                                              \
        {                                                                           \
            FAPEX_BOARDIFC_Ioctl( fd_bd, FAPEX_IOCCOM_PLATFORM_SUPPORT_EXT, (FAPEXIOCARG3_T)NULL );    \
        }

#ifdef __cplusplus
}
#endif
#endif /* !__LINUX__ */

#endif /* !_FAPEX_ABST_LIBBOARDIFC_H */

