/*!
*************************************************************************
**
** \file    fapex/abst_libhdmicec.h
**
** \brief   abstraction library header file (HDMI-CEC driver)
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#ifndef _FAPEX_ABST_LIBHDMICEC_H
#define _FAPEX_ABST_LIBHDMICEC_H

#include "fapex/abst_lib.h"
#include "fapex/abst_error.h"
#include "fapex/abst_libhdmi.h"

/*
*************************************************************************
*************************************************************************
** Defines and Macros
*************************************************************************
*************************************************************************/

/*! \brief HDMI CEC is sending the message */
#define FAPEX_HDMICEC_STATUS_ERR_BUSY                 (FAPEX_HDMI_STATUS_ERR_BASE - 51)
/*! \brief HDMI CEC error of message buffer empty */
#define FAPEX_HDMICEC_STATUS_ERR_MESSAGE_BUFFER_EMPTY (FAPEX_HDMI_STATUS_ERR_BASE - 55)
/*! \brief HDMI CEC error of receiving parameter */
#define FAPEX_HDMICEC_STATUS_ERR_RECEIVE_PARAMETER    (FAPEX_HDMI_STATUS_ERR_BASE - 56)
/*! \brief HDMI CEC error of sending parameter */
#define FAPEX_HDMICEC_STATUS_ERR_SEND_PARAMETER       (FAPEX_HDMI_STATUS_ERR_BASE - 57)

/* Message Descriptions for System Standby */
#define FAPEX_HDMICEC_STANDBY                       (0x36)      //! Opcode <Standby>
/* Message Descriptions for Tuner Control */
#define FAPEX_HDMICEC_GIVE_TUNER_DEVICE_STATUS      (0x08)      //! Opcode <Give Tuner Device Status>
#define FAPEX_HDMICEC_SELECT_ANALOGUE_SERVICE       (0x92)      //! Opcode <Select Analogue Service>
#define FAPEX_HDMICEC_SELECT_DIGITAL_SERVICE        (0x93)      //! Opcode <Select Digital Service>
#define FAPEX_HDMICEC_TUNER_DEVICE_STATUS           (0x07)      //! Opcode <Tuner Device Status>
#define FAPEX_HDMICEC_TUNER_STEP_DECREMENT          (0x06)      //! Opcode <Tuner Step Decrement>
#define FAPEX_HDMICEC_TUNER_STEP_INCREMENT          (0x05)      //! Opcode <Tuner Step Increment>
/* Message Descriptions for OSD Display */
#define FAPEX_HDMICEC_SET_OSD_STRING                (0x64)      //! Opcode <Set OSD String>
/* Message Descriptions for Device Menu Control */
#define FAPEX_HDMICEC_MENU_REQUEST                  (0x8D)      //! Opcode <Menu Request>
#define FAPEX_HDMICEC_MENU_STATUS                   (0x8E)      //! Opcode <Menu Status>
#define FAPEX_HDMICEC_USER_CONTROL_PRESSED          (0x44)      //! Opcode <User Control Pressed>
                                                                /* This message is also used in the RC Passthrough */
#define FAPEX_HDMICEC_USER_CONTROL_RELEASED         (0x45)      //! Opcode <User Control Released>
                                                                /* This message is also used in the RC Passthrough */
/* Message Descriptions for Give Device Status */
#define FAPEX_HDMICEC_GIVE_DEVICE_POWER_STATUS      (0x8F)      //! Opcode <Give Device Power Status>
#define FAPEX_HDMICEC_REPORT_POWER_STATUS           (0x90)      //! Opcode <Report Power Status>
/* Message Descriptions for System Information */
#define FAPEX_HDMICEC_CEC_VERSION                   (0x9e)      //! Opcode <CEC Version>
#define FAPEX_HDMICEC_GET_CEC_VERSION               (0x9F)      //! Opcode <Get CEC Version>
#define FAPEX_HDMICEC_GIVE_PHYSICAL_ADDRESS         (0x83)      //! Opcode <Give Physical Address>
#define FAPEX_HDMICEC_GET_MENU_LANGUAGE             (0x91)      //! Opcode <Get Menu Language>
#define FAPEX_HDMICEC_REPORT_PHYSICAL_ADDRESS       (0x84)      //! Opcode <Report Physical Address>
#define FAPEX_HDMICEC_POLLING_MESSAGE               (0xffff)    //! Opcode <Polling message>
#define FAPEX_HDMICEC_SET_MENU_LANGUAGE             (0x32)      //! Opcode <Set Menu Language>
/* Message Descriptions for General Protocol */
#define FAPEX_HDMICEC_FEATURE_ABORT                 (0x00)      //! Opcode <Feature Abort>
#define FAPEX_HDMICEC_ABORT                         (0xff)      //! Opcode <Abort>messeage for debug
/* Message Descriptions for One Touch Play */
#define FAPEX_HDMICEC_ACTIVE_SOURCE                 (0x82)      //! Opcode <Active Source>
                                                                /* This message is also used in the Routing Control Feature */
#define FAPEX_HDMICEC_IMAGE_VIEW_ON                 (0x04)      //! Opcode <Image View On>
#define FAPEX_HDMICEC_TEXT_VIEW_ON                  (0x0D)      //! Opcode <Text View On>
/* Message Descriptions for Routing Control */
#define FAPEX_HDMICEC_INACTIVE_SOURCE               (0x9D)      //! Opcode <Inactive Source>
#define FAPEX_HDMICEC_REQUEST_ACTIVE_SOURCE         (0x85)      //! Opcode <Request Active Source>
#define FAPEX_HDMICEC_ROUTING_CHANGE                (0x80)      //! Opcode <Routing Change>
#define FAPEX_HDMICEC_ROUTING_INFORMATION           (0x81)      //! Opcode <Routing Information>
#define FAPEX_HDMICEC_SET_STREAM_PATH               (0x86)      //! Opcode <Set Stream Path>

/* Another Message Descriptions */
#define FAPEX_HDMICEC_DEVICE_VENDOR_ID              (0x87)      //! Opcode <Device Vender Id>
#define FAPEX_HDMICEC_GIVE_DEVICE_VENDOR_ID         (0x8C)      //! Opcode <Give Device Vender Id>
#define FAPEX_HDMICEC_SET_SYSTEM_AUDIO_MODE         (0x72)      //! Opcode <Set System Audio Mode>

/* Communication status for CEC */
#define FAPEX_HDMICEC_AWAITING_SENDING    (0)      //! state of awaiting sending
#define FAPEX_HDMICEC_DURING_SENDING      (1)      //! state of during sending
#define FAPEX_HDMICEC_SENDING_COMPLETE    (2)      //! state of sending complete
#define FAPEX_HDMICEC_SENDING_ERROR       (3)      //! state of sending error
#define FAPEX_HDMICEC_AWAITING_RECEIVING  (0)      //! state of awaiting receiving
#define FAPEX_HDMICEC_DURING_RECEIVING    (1)      //! state of during receving
#define FAPEX_HDMICEC_RECEPTION_COMPLETE  (2)      //! state of reception complete
#define FAPEX_HDMICEC_RECEPTION_ERROR     (3)      //! state of reception error

/*!
*************************************************************************
** \brief Logic address of an HDMI device of a connection point.
**
** This enumeration contains the IDs of the supported logical address.
**
** \note The logical address(s) actual available is restricted by the
**       HDMI device which was connected
**
*************************************************************************
*/

typedef enum {
    FAPEX_HDMICEC_LOGICAL_ADDRESS_TV = 0,                  //!< TV
    FAPEX_HDMICEC_LOGICAL_ADDRESS_RECORDING_DEVICE1,       //!< Recording device1
    FAPEX_HDMICEC_LOGICAL_ADDRESS_RECORDING_DEVICE2,       //!< Recording device2
    FAPEX_HDMICEC_LOGICAL_ADDRESS_TUNER1,                  //!< Tuner1
    FAPEX_HDMICEC_LOGICAL_ADDRESS_PLAYBACK_DEVICE1,        //!< Playback device1
    FAPEX_HDMICEC_LOGICAL_ADDRESS_AUDIO_SYSTEM,            //!< Audio System
    FAPEX_HDMICEC_LOGICAL_ADDRESS_TUNER2,                  //!< Tuner2
    FAPEX_HDMICEC_LOGICAL_ADDRESS_TUNER3,                  //!< Tuner3
    FAPEX_HDMICEC_LOGICAL_ADDRESS_PLAYBACK_DEVICE2,        //!< Playback device2
    FAPEX_HDMICEC_LOGICAL_ADDRESS_RECORDING_DEVICE3,       //!< Recording device3
    FAPEX_HDMICEC_LOGICAL_ADDRESS_TUNER4,                  //!< Tuner4
    FAPEX_HDMICEC_LOGICAL_ADDRESS_PLAYBACK_DEVICE3,        //!< Playback device3
    FAPEX_HDMICEC_LOGICAL_ADDRESSS_RESERVED1,              //!< Reserved1
    FAPEX_HDMICEC_LOGICAL_ADDRESSS_RESERVED2,              //!< Reserved2
    FAPEX_HDMICEC_LOGICAL_ADDRESSS_FREE_USE,               //!< Free Use
    FAPEX_HDMICEC_LOGICAL_ADDRESSS_UNREGISTERED_BROADCAST  //!< Unregistered(as initiator address),Broadcast(as definition address)
} FAPEX_HDMICEC_LogicalAddressEnumT;

/*
*************************************************************************
*************************************************************************
** Data types
*************************************************************************
*************************************************************************/

/*!
*************************************************************************
** \brief Set/Get the Give Tuner Device Status parameters for the HDMI driver.
**
** This data structure covers the Give Tuner Device Status parameters which need to be specified
** when an information on the CEC is set and get.
**
*************************************************************************
*/
typedef struct {
    uint8_t             Status_Request;           /*!<
                                                       - 1 : On
                                                       - 2 : Of
                                                       - 3 : Once */
} FAPEX_HDMICEC_GiveTunerDeviceStatusParamsT;

/*!
*************************************************************************
** \brief Set/Get the Select Analogue Service parameters for the HDMI driver.
**
** This data structure covers the Select Analogue Service parameters which need to be specified
** when an information on the CEC is set and get.
**
*************************************************************************
*/
typedef struct {
    uint8_t             Analogue_Broadcast_Type;    /*!<
                                                         - 0x00 : Cable
                                                         - 0x01 : Satellite
                                                         - 0x02 : Terrestrial */
    uint8_t             Analogue_Frequency[2];      //!< 0x0000<N<0xFFFF
    uint8_t             Broadcast_System;           //!< 0<=N<=31 See CEC Table 28
} FAPEX_HDMICEC_SelectAnalogueServiceParamsT;

/*!
*************************************************************************
** \brief Set/Get the Select Digital Service parameters for the HDMI driver.
**
** This data structure covers the Select Digital Service parameters which need to be specified
** when an information on the CEC is set and get.
**
*************************************************************************
*/
typedef struct {
    uint8_t             Digital_Service_Identification[7];    //!< [Service Identification Method:1bit],[Digital Broadcast System:7bit],[Service Identification:6byte]
} FAPEX_HDMICEC_SelectDigitalServiceParamsT;

/*!
*************************************************************************
** \brief Set/Get the Tuner Device Status parameters for the HDMI driver.
**
** This data structure covers the Tuner Device Status parameters which need to be specified
** when an information on the CEC is set and get.
**
*************************************************************************
*/
typedef struct {
    uint8_t             deviceType;              //!< analog : 0 , digital : 1
    uint8_t             Tuner_Device_Info[8];    //!< [Tuner Device Info], 5 bytes (analogue service);8 bytes (digital service)
} FAPEX_HDMICEC_TunerDeviceStatusParamsT;

/*!
*************************************************************************
** \brief Set/Get the Set OSD String parameters for the HDMI driver.
**
** This data structure covers the Set OSD String parameters which need to be specified
** when an information on the CEC is set and get.
**
*************************************************************************
*/
typedef struct {
    uint8_t             Display_Control;        //!< Bit 7 Bit 6 = {0,0}{0,1}{1,0}{1,1}  bit 5 - bit 0 = 0
    uint8_t             OSD_String[13+1];       //!< N {[ASCII]}, 1<=N<=13
    uint8_t             StringLength;           //!< string length 1<=N<=13
} FAPEX_HDMICEC_SetOSDStringParamsT;

/*!
*************************************************************************
** \brief Set/Get the Menu Request parameters for the HDMI driver.
**
** This data structure covers the Menu Request parameters which need to be specified
** when an information on the CEC is set and get.
**
*************************************************************************
*/
typedef struct {
    uint8_t             Menu_Request_Type;        /*!<
                                                       - 0 : Activate
                                                       - 1 : Deactivate
                                                       - 2 : Query */
} FAPEX_HDMICEC_MenuRequestParamsT;

/*!
*************************************************************************
** \brief Set/Get the Menu Status parameters for the HDMI driver.
**
** This data structure covers the Menu Status parameters which need to be specified
** when an information on the CEC is set and get.
**
*************************************************************************
*/
typedef struct {
    uint8_t             Menu_State;        /*!<
                                                - 0 : Activated
                                                - 1 : Deactivated */
} FAPEX_HDMICEC_MenuStatusParamsT;

/*!
*************************************************************************
** \brief Set/Get the User Control Pressed parameters for the HDMI driver.
**
** This data structure covers the User Control Pressed parameters which need to be specified
** when an information on the CEC is set and get.
**
*************************************************************************
*/
typedef struct {
    uint8_t             UI_Command;        //!< 0x00<=n<=0xFF (n is defined in Table 27)
} FAPEX_HDMICEC_UserControlPressedParamsT;

/*!
*************************************************************************
** \brief Set/Get the Report Power Status parameters for the HDMI driver.
**
** This data structure covers the Report Power Status parameters which need to be specified
** when an information on the CEC is set and get.
**
*************************************************************************
*/
typedef struct {
    uint8_t             Power_Status;        /*!<
                                                  - 0x00 : On
                                                  - 0x01 : Standby
                                                  - 0x02 : In transition Standby to On
                                                  - 0x03 : In transition On to Standby */
} FAPEX_HDMICEC_ReportPowerStatusParamsT;

/*!
*************************************************************************
** \brief Set/Get the CEC version parameters for the HDMI driver.
**
** This data structure covers the CEC version parameters which need to be specified
** when an information on the CEC is set and get.
**
*************************************************************************
*/
typedef struct {
    uint8_t             CEC_Version;           /*!<
                                                    - 0x00 : Version 1.1
                                                    - 0x01 : Version 1.2
                                                    - 0x02 : Version 1.2a
                                                    - 0x03 : Version 1.3
                                                    - 0x04 : Version 1.3a */
} FAPEX_HDMICEC_CECVersionParamsT;

/*!
*************************************************************************
** \brief Set/Get the Report Physical Address parameters for the HDMI driver.
**
** This data structure covers the Report Physical Address parameters which need to be specified
** when an information on the CEC is set and get.
**
*************************************************************************
*/
typedef struct {
    uint8_t             Physical_Address[2];   //!< 4{[Port ID]}
    uint8_t             Device_Type;           /*!<
                                                    - 0 : TV
                                                    - 1 : Recording Device
                                                    - 2 : Reserved
                                                    - 3 : Tuner
                                                    - 4 : Playback Device
                                                    - 5 : Audio System */
} FAPEX_HDMICEC_ReportPhysicalAddressParamsT;

/*!
*************************************************************************
** \brief Set/Get the Set Menu Language parameters for the HDMI driver.
**
** This data structure covers the Set Menu Language parameters which need to be specified
** when an information on the CEC is set and get.
**
*************************************************************************
*/
typedef struct {
    uint8_t             Language[3];           //!< 3 {[ASCII]} as defined in ISO/FDIS 639-2[ref 1n]
} FAPEX_HDMICEC_SetMenuLanguageParamsT;

/*!
*************************************************************************
** \brief Set/Get the Feature Abort parameters for the HDMI driver.
**
** This data structure covers the General Protocol parameters which need to be specified
** when an information on the CEC is set and get.
**
*************************************************************************
*/
typedef struct {
    uint8_t             Feature_Opcode;           //!< 0x00 <= N <= 0xFF(n is defined in CEC Table 7 to CEC Table 23)
    uint8_t             Abort_Reason;             /*!<
                                                       - 0 : Unrecognized opcode
                                                       - 1 : Not in correct mode to respond
                                                       - 2 : Cannot provide source 
                                                       -   : Invalid operand
                                                       -   : Refused */
} FAPEX_HDMICEC_FeatureAbortParamsT;

/*!
*************************************************************************
** \brief Set/Get the Active Source parameters for the HDMI driver.
**
** This data structure covers the Active Source parameters which need to be specified
** when an information on the CEC is set and get.
**
*************************************************************************
*/
typedef struct {
    uint8_t             Physical_Address[2];   //!< 4{[Port ID]}
} FAPEX_HDMICEC_ActiveSourceParamsT;

/*!
*************************************************************************
** \brief Set/Get the Inactive Source parameters for the HDMI driver.
**
** This data structure covers the Inactive Source parameters which need to be specified
** when an information on the CEC is set and get.
**
*************************************************************************
*/
typedef struct {
    uint8_t             Physical_Address[2];   //!< 4{[Port ID]}
} FAPEX_HDMICEC_InactiveSourceParamsT;

/*!
*************************************************************************
** \brief Set/Get the Routing Change parameters for the HDMI driver.
**
** This data structure covers the Routing Change parameters which need to be specified
** when an information on the CEC is set and get.
**
*************************************************************************
*/
typedef struct {
    uint8_t             Original_Address[2];   //!< 4{[Port ID]}
    uint8_t             New_Address[2];        //!< 4{[Port ID]}
} FAPEX_HDMICEC_RoutingChangeParamsT;

/*!
*************************************************************************
** \brief Set/Get the Routing Information parameters for the HDMI driver.
**
** This data structure covers the Routing Information parameters which need to be specified
** when an information on the CEC is set and get.
**
*************************************************************************
*/
typedef struct {
    uint8_t             Physical_Address[2];   //!< 4{[Port ID]}
} FAPEX_HDMICEC_RoutingInformationParamsT;

/*!
*************************************************************************
** \brief Set/Get the Set Stream Path parameters for the HDMI driver.
**
** This data structure covers the Set Stream Path parameters which need to be specified
** when an information on the CEC is set and get.
**
*************************************************************************
*/
typedef struct {
    uint8_t             Physical_Address[2];   //!< 4{[Port ID]}
} FAPEX_HDMICEC_SetStreamPathParamsT;

/*!
*************************************************************************
** \brief Set/Get CEC parameters for the HDMI driver.
**
** This data structure covers all parameters which need to be specified
** when an information on the CEC is set and get.
**
*************************************************************************
*/
typedef struct {
    uint8_t                                         message_type;                   //!< The value "0" is single(directly) message. The value "1" is broadcaessage.
    FAPEX_HDMICEC_GiveTunerDeviceStatusParamsT     cecGiveTunerDeviceStatusParams; //!< The sturucture of Tuner Device Status
    FAPEX_HDMICEC_SelectAnalogueServiceParamsT     cecSelectAnalogueServiceParams; //!< The sturucture of Set Analogue Timer
    FAPEX_HDMICEC_SelectDigitalServiceParamsT      cecSelectDigitalServiceParams;  //!< The sturucture of Select Analogue Service
    FAPEX_HDMICEC_TunerDeviceStatusParamsT         cecTunerDeviceStatusParams;     //!< The sturucture of Tuner Device Status
    FAPEX_HDMICEC_SetOSDStringParamsT              cecSetOSDStringParams;          //!< The sturucture of Set OSD String
    FAPEX_HDMICEC_MenuRequestParamsT               cecMenuRequestParams;           //!< The sturucture of Menu Request
    FAPEX_HDMICEC_MenuStatusParamsT                cecMenuStatusParams;            //!< The sturucture of Menu Status
    FAPEX_HDMICEC_UserControlPressedParamsT        cecUserControlPressedParams;    //!< The sturucture of UserControlPressed for Device Menu Control and Re Control Pass Through
    FAPEX_HDMICEC_ReportPowerStatusParamsT         cecReportPowerStatusParams;     //!< The sturucture of Report Power Status
    FAPEX_HDMICEC_CECVersionParamsT                cecCECVersionParams;            //!< The sturucture of cec version
    FAPEX_HDMICEC_ReportPhysicalAddressParamsT     cecReportPhysicalAddressParams; //!< The sturucture of Report Physical Address
    FAPEX_HDMICEC_SetMenuLanguageParamsT           cecSetMenuLanguageParams;       //!< The sturucture of Set Menu Language
    FAPEX_HDMICEC_FeatureAbortParamsT              cecFeatureAbortParams;          //!< The sturucture of Feature Abort
    FAPEX_HDMICEC_ActiveSourceParamsT              cecActiveSourceParams;          //!< The sturucture of Active Source on One Touch Play and Routing Contr
    FAPEX_HDMICEC_InactiveSourceParamsT            cecInactiveSourceParams;        //!< The sturucture of Inactive Source
    FAPEX_HDMICEC_RoutingChangeParamsT             cecRoutingChangeParams;         //!< The sturucture of Routing Change
    FAPEX_HDMICEC_RoutingInformationParamsT        cecRoutingInformationParams;    //!< The sturucture of Routing Information
    FAPEX_HDMICEC_SetStreamPathParamsT             cecSetStreamPathParams;         //!< The sturucture of Set Stream Path
} FAPEX_HDMICEC_MessageParamsT;

/*!
*************************************************************************
** \brief message parameters for the HDMICEC driver
**
** The related ioctl command type :
** \ref hdmicec_cmd_set_message "FAPEX_IOCCOM_HDMICEC_SET_MESSAGE",
** \ref hdmicec_cmd_get_message "FAPEX_IOCCOM_HDMICEC_GET_MESSAGE"
**
** \sa FAPEX_HDMICEC_Ioctl
*************************************************************************/
typedef struct {
    /*!
        initiator logical address.
    */
    uint32_t    initiatorLogicalAddr;
    /*!
        followr logical address.
    */
    uint32_t    followerLogicalAddr;
    /*!
        operation code.
    */
    uint32_t    opcodeName;
    /*!
        CEC message parameters
    */
    FAPEX_HDMICEC_MessageParamsT   message;
    /*!
        the number of remained received message
    */
    uint32_t    remainRecMsgNum;

} FAPEX_HDMICEC_MessageParamStrT;


/*!
*************************************************************************
** \brief ioctl specific command for the HDMI-CEC driver
*************************************************************************/
#define FAPEX_IOCCOM_HDMICEC_GET_SEND_STATUS        _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_HDMICEC + 0, uint32_t)
#define FAPEX_IOCCOM_HDMICEC_GET_RECEIVE_STATUS     _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_HDMICEC + 1, uint32_t)
#define FAPEX_IOCCOM_HDMICEC_CHECK_SEND_STATUS      _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_HDMICEC + 2, uint32_t)
#define FAPEX_IOCCOM_HDMICEC_CHECK_RECEIVE_STATUS   _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_HDMICEC + 3, uint32_t)
#define FAPEX_IOCCOM_HDMICEC_SET_MESSAGE            _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_HDMICEC + 4, FAPEX_HDMICEC_MessageParamStrT)
#define FAPEX_IOCCOM_HDMICEC_GET_MESSAGE            _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_HDMICEC + 4, FAPEX_HDMICEC_MessageParamStrT)
#define FAPEX_IOCCOM_HDMICEC_SEND_MESSAGE           _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_HDMICEC + 5, int32_t)
#define FAPEX_IOCCOM_HDMICEC_RECEIVE_MESSAGE        _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_HDMICEC + 6, int32_t)
#define FAPEX_IOCCOM_HDMICEC_SET_LOGICAL_ADDRESS    _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_HDMICEC + 7, uint32_t)
#define FAPEX_IOCCOM_HDMICEC_GET_LOGICAL_ADDRESS    _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_HDMICEC + 8, uint32_t)


/*
*************************************************************************
*************************************************************************
** prototype for application
*************************************************************************
*************************************************************************/
#ifndef __LINUX__
#include "fapex/abst_types.h"

#ifdef __cplusplus
extern "C" {
#endif

FAPEXOPENRET_T FAPEX_HDMICEC_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr);
FAPEXRELRET_T  FAPEX_HDMICEC_Close(FAPEXRELARG1_T id);
FAPEXIOCRET_T  FAPEX_HDMICEC_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg);

#ifdef __cplusplus
}
#endif
#endif /* !__LINUX__ */


#endif /* !_FAPEX_ABST_LIBHDMICEC_H */

