/*!
*************************************************************************
**
** \file    fapex/abst_libi2c.h
**
** \brief   abstraction library header file (I2C driver)
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#ifndef _FAPEX_ABST_LIBI2C_H
#define _FAPEX_ABST_LIBI2C_H

#include "fapex/abst_lib.h"
#include "fapex/abst_error.h"

/*
*************************************************************************
*************************************************************************
** Defines and Macros
*************************************************************************
*************************************************************************/

    #define FAPEX_I2C0            (uint32_t) 0    //!< I2C0 hardware block
    #define FAPEX_I2C1            (uint32_t) 1    //!< I2C1 hardware block
    #define FAPEX_I2C2            (uint32_t) 2    //!< I2C2 hardware block

/*
*************************************************************************
** Standard error codes for the driver
*************************************************************************/
/*! FAPex I2C error code base value */
#define FAPEX_I2C_ERR_BASE            (int32_t)(FAPEX_I2C_MODULE_BASE + FAPEX_RET_ERR_BASE)

/*! Bad parameter passed. */
#define FAPEX_I2C_ERR_BAD_PARAMETER   (int32_t)(FAPEX_I2C_ERR_BASE + FAPEX_ERR_BAD_PARAMETER)

/*!  Memory allocation failed. */
#define FAPEX_I2C_ERR_OUT_OF_MEMORY   (int32_t)(FAPEX_I2C_ERR_BASE + FAPEX_ERR_OUT_OF_MEMORY)

/*
*************************************************************************
*************************************************************************
** Data types
*************************************************************************
*************************************************************************/

    /*!
    *******************************************************************************
    **
    ** \brief I2C datarate speed modes.
    **
    ** This enumeration specifies the speed of the I2C bus.
    **
    ** \sa    FAPEX_I2C_OpenParamsT
    **
    *******************************************************************************
    */
    typedef enum
    {
        FAPEX_I2C_100KBPS = (int32_t)1, //!< 100kHz datarate.
        FAPEX_I2C_400KBPS = (int32_t)2  //!< 400kHz datarate.
    } FAPEX_I2C_SpeedT;

    /*!
    ****************************************************************************
    **
    ** \brief Protocol modes.
    **
    ** This enumeration specifies which protocol shall be used.
    ** #FAPEX_I2C_RESTART_PROTOCOL is used in case a restart has to be sent after
    ** the addressing sequence.
    **
    ** \sa    FAPEX_I2C_SetProtocol()
    **
    ****************************************************************************
    */
    typedef enum
    {
        FAPEX_I2C_COMMON_PROTOCOL  = (int32_t)0,  //!< Common protocol.
        FAPEX_I2C_RESTART_PROTOCOL = (int32_t)1   //!< Protocol with restart.
    } FAPEX_I2C_ProtocolT;

    /*!
    ****************************************************************************
    **
    ** \brief I2C types.
    **
    ** This enumeration specifies whether the I2C bus shall be run in HW mode
    ** or in SW mode.
    **
    ** \sa    FAPEX_I2C_OpenParamsT
    **
    ****************************************************************************
    */
    typedef enum
    {
        FAPEX_I2C_HARDWARE = (int32_t)0,  //!< I2C hardware driver.
        FAPEX_I2C_SOFTWARE = (int32_t)1   //!< I2C software driver.
    } FAPEX_I2C_TypeT;

    /*!
    ****************************************************************************
    **
    ** \brief I2C operation modes.
    **
    ** This enumeration covers all possible modes of operation. Software mode
    ** is determined by #FAPEX_I2C_TypeT.
    **
    ** \sa    FAPEX_I2C_SetOperationMode()
    **
    ****************************************************************************
    */
    typedef enum
    {
        FAPEX_I2C_GENERIC_MASTER_MODE = (int32_t)0, //!< Generic master mode.
        FAPEX_I2C_GENERIC_SLAVE_MODE  = (int32_t)1, //!< Generic slave mode.
        FAPEX_I2C_AUTO_MASTER_MODE    = (int32_t)2  //!< Auto master mode.
    } FAPEX_I2C_OperationModeT;


/*!
*************************************************************************
** \brief open parameters for the I2C driver
**
** \sa FAPEX_I2C_Open
*************************************************************************/
typedef struct {
        uint32_t              blockIndex; //!< The I2C HW block to be used. #FAPEX_I2C0, #FAPEX_I2C1, #FAPEX_I2C2
         int32_t                   speed; //!< The speed to be used by I2C. #FAPEX_I2C_100KBPS, #FAPEX_I2C_400KBPS
         int32_t                protocol; //!< The protocol to be used. #FAPEX_I2C_COMMON_PROTOCOL, #FAPEX_I2C_RESTART_PROTOCOL
        uint32_t            slaveAddress; //!< The physical address of the slave device.
         int32_t                    type; //!< The type of I2C driver. #FAPEX_I2C_HARDWARE, #FAPEX_I2C_SOFTWARE
        uint32_t                 timeOut; //!< I2C minimum timeout is 100 ms
} FAPEX_I2C_OpenParamStrT;

/*!
*************************************************************************
** \brief read/write parameters for the I2C driver
**
** The related ioctl command type :
** \ref i2c_cmd_read  "FAPEX_IOCCOM_I2C_READ"
** \ref i2c_cmd_write "FAPEX_IOCCOM_I2C_WRITE"
**
** \sa FAPEX_I2C_Ioctl 
*************************************************************************/
typedef struct {
        uint8_t           msSlaveAddress; //!< Slave address of external I2C device.
        uint8_t*               txDataPtr; //!< Pointer to data to be transmitted to an I2C device.
        uint32_t            txDataLength; //!< Length of data to be transmitted in bytes.
        uint8_t*               rxDataPtr; //!< Pointer to data to be received from I2C device.
        uint32_t            rxDataLength; //!< Length of data to be received from an I2C device.
} FAPEX_I2C_RWParamStrT;


/*!
*************************************************************************
** \brief ioctl specific command for the I2C driver
*************************************************************************/
#define FAPEX_IOCCOM_I2C_OPEN           _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_I2C + 0, FAPEX_I2C_OpenParamStrT)
#define FAPEX_IOCCOM_I2C_SET_PROTOCOL   _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_I2C + 1, int32_t)
#define FAPEX_IOCCOM_I2C_READ           _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_I2C + 2, FAPEX_I2C_RWParamStrT)
#define FAPEX_IOCCOM_I2C_WRITE          _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_I2C + 2, FAPEX_I2C_RWParamStrT)


/*
*************************************************************************
*************************************************************************
** prototype for application
*************************************************************************
*************************************************************************/
#ifndef __LINUX__
#include "fapex/abst_types.h"

#ifdef __cplusplus
extern "C" {
#endif

FAPEXOPENRET_T FAPEX_I2C_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr);
FAPEXRELRET_T  FAPEX_I2C_Close(FAPEXRELARG1_T id);
FAPEXIOCRET_T  FAPEX_I2C_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg);

#ifdef __cplusplus
}
#endif
#endif /* !__LINUX__ */

#endif /* !_FAPEX_ABST_LIBI2C_H */

