/*!
*************************************************************************
**
** \file    fapex/abst_libvienc.h
**
** \brief   abstraction library header file (Video Output Encoder driver)
**
** \attention  This program is provided as is. You can redistribute it and/or
** modify it. Fujitsu Semiconductor Limited accepts no responsibility or
** liability for any errors or omissions.
**
** Copyright (C) 2008-2010 Fujitsu Semiconductor Limited, All Rights Reserved.
**
*************************************************************************
*/

#ifndef _FAPEX_ABST_LIBVIENC_H
#define _FAPEX_ABST_LIBVIENC_H

#include "fapex/abst_lib.h"
#include "fapex/abst_error.h"

/*
*************************************************************************
*************************************************************************
** Defines and Macros
*************************************************************************
*************************************************************************/

/*
*************************************************************************
** Standard error codes for the driver
*************************************************************************/
/*! FAPex VIENC error code base value */
#define FAPEX_VIENC_ERR_BASE             (FAPEX_VIENC_MODULE_BASE + FAPEX_RET_ERR_BASE)

/*! Bad parameter passed. */
#define FAPEX_VIENC_ERR_BAD_PARAMETER    (FAPEX_VIENC_ERR_BASE + FAPEX_ERR_BAD_PARAMETER)

/*! Memory allocation failed. */
#define FAPEX_VIENC_ERR_OUT_OF_MEMORY    (FAPEX_VIENC_ERR_BASE + FAPEX_ERR_OUT_OF_MEMORY)

/*! Invalid status. */
#define FAPEX_VIENC_ERR_INVALID_STATUS   (FAPEX_VIENC_ERR_BASE + FAPEX_ERR_INVALID_HANDLE)

/*! Semaphore could not be created. */
#define FAPEX_VIENC_ERR_SEMAPHORE_CREATE (FAPEX_VIENC_ERR_BASE + FAPEX_ERR_SEMAPHORE_CREATE)


/*!
*******************************************************************************
** \brief Enumerated type for analog encoder system modes
*******************************************************************************
*/
#define FAPEX_VIENC_SYSTEM_MODE_FREQ_MSK            0x0000000F  /*!< system frequency mode mask */
#define FAPEX_VIENC_SYSTEM_MODE_FREQ2397            0x00000001  /*!< 23.97 Hz */
#define FAPEX_VIENC_SYSTEM_MODE_FREQ2400            0x00000002  /*!< 24    Hz */
#define FAPEX_VIENC_SYSTEM_MODE_FREQ2500            0x00000003  /*!< 25    Hz */
#define FAPEX_VIENC_SYSTEM_MODE_FREQ2997            0x00000004  /*!< 29.97 Hz */
#define FAPEX_VIENC_SYSTEM_MODE_FREQ3000            0x00000005  /*!< 30    Hz */
#define FAPEX_VIENC_SYSTEM_MODE_FREQ5000            0x00000006  /*!< 50    Hz */
#define FAPEX_VIENC_SYSTEM_MODE_FREQ5994            0x00000007  /*!< 59.94 Hz */
#define FAPEX_VIENC_SYSTEM_MODE_FREQ6000            0x00000008  /*!< 60    Hz */
#define FAPEX_VIENC_SYSTEM_MODE_PROGRESSIVE_MSK     0x00000010  /*!< system progressive mode mask */
#define FAPEX_VIENC_SYSTEM_MODE_PROGRESSIVE         0x00000010  /*!< Progressive */
#define FAPEX_VIENC_SYSTEM_MODE_INTERLACE           0x00000000  /*!< Interlace   */
#define FAPEX_VIENC_SYSTEM_MODE_LCD                 0x00010000  /*!< system LCD mode flag */
#define FAPEX_VIENC_SYSTEM_MODE_FSIZE_MSK           0x0001F000  /*!< system frame size mask */
#define FAPEX_VIENC_SYSTEM_MODE_FSIZE_1920x1080     0x00007000  /*!< 1920 x 1080 */
#define FAPEX_VIENC_SYSTEM_MODE_FSIZE_1360x768      0x00006000  /*!< 1360 x  768 */
#define FAPEX_VIENC_SYSTEM_MODE_FSIZE_1280x720      0x00005000  /*!< 1280 x  720 */
#define FAPEX_VIENC_SYSTEM_MODE_FSIZE_1024x768      0x00004000  /*!< 1024 x  768 */
#define FAPEX_VIENC_SYSTEM_MODE_FSIZE_800x600       0x00003000  /*!<  800 x  600 */
#define FAPEX_VIENC_SYSTEM_MODE_FSIZE_720x576       0x00002000  /*!<  720 x  576 */
#define FAPEX_VIENC_SYSTEM_MODE_FSIZE_720x480       0x00001000  /*!<  720 x  480 */
#define FAPEX_VIENC_SYSTEM_MODE_FSIZE_640x480       0x00000000  /*!<  640 x  480 */
#define FAPEX_VIENC_SYSTEM_MODE_HDMI_FREQ_MSK       0x00000F00  /*!< system HDMI frequency mode mask */
#define FAPEX_VIENC_SYSTEM_MODE_HDMI_FREQ24         0x00000000  /*!< 23.97, 24 Hz */
#define FAPEX_VIENC_SYSTEM_MODE_HDMI_FREQ25         0x00000100  /*!< 25        Hz */
#define FAPEX_VIENC_SYSTEM_MODE_HDMI_FREQ30         0x00000200  /*!< 29.97, 30 Hz */
#define FAPEX_VIENC_SYSTEM_MODE_HDMI_FREQ50         0x00000300  /*!< 50        Hz */
#define FAPEX_VIENC_SYSTEM_MODE_HDMI_FREQ60         0x00000400  /*!< 59.54, 60 Hz */
#define FAPEX_VIENC_SYSTEM_MODE_HDMI_MSK            (FAPEX_VIENC_SYSTEM_MODE_FSIZE_MSK | FAPEX_VIENC_SYSTEM_MODE_PROGRESSIVE_MSK | FAPEX_VIENC_SYSTEM_MODE_HDMI_FREQ_MSK)
#define FAPEX_VIENC_SYSTEM_MODE_HDMI_1920x1080x24P  (FAPEX_VIENC_SYSTEM_MODE_FSIZE_1920x1080 | FAPEX_VIENC_SYSTEM_MODE_PROGRESSIVE | FAPEX_VIENC_SYSTEM_MODE_HDMI_FREQ24)
#define FAPEX_VIENC_SYSTEM_MODE_HDMI_1920x1080x25P  (FAPEX_VIENC_SYSTEM_MODE_FSIZE_1920x1080 | FAPEX_VIENC_SYSTEM_MODE_PROGRESSIVE | FAPEX_VIENC_SYSTEM_MODE_HDMI_FREQ25)
#define FAPEX_VIENC_SYSTEM_MODE_HDMI_1920x1080x30P  (FAPEX_VIENC_SYSTEM_MODE_FSIZE_1920x1080 | FAPEX_VIENC_SYSTEM_MODE_PROGRESSIVE | FAPEX_VIENC_SYSTEM_MODE_HDMI_FREQ30)
#define FAPEX_VIENC_SYSTEM_MODE_HDMI_1920x1080x50I  (FAPEX_VIENC_SYSTEM_MODE_FSIZE_1920x1080 | FAPEX_VIENC_SYSTEM_MODE_INTERLACE   | FAPEX_VIENC_SYSTEM_MODE_HDMI_FREQ50)
#define FAPEX_VIENC_SYSTEM_MODE_HDMI_1920x1080x50P  (FAPEX_VIENC_SYSTEM_MODE_FSIZE_1920x1080 | FAPEX_VIENC_SYSTEM_MODE_PROGRESSIVE | FAPEX_VIENC_SYSTEM_MODE_HDMI_FREQ50)
#define FAPEX_VIENC_SYSTEM_MODE_HDMI_1920x1080x60I  (FAPEX_VIENC_SYSTEM_MODE_FSIZE_1920x1080 | FAPEX_VIENC_SYSTEM_MODE_INTERLACE   | FAPEX_VIENC_SYSTEM_MODE_HDMI_FREQ60)
#define FAPEX_VIENC_SYSTEM_MODE_HDMI_1920x1080x60P  (FAPEX_VIENC_SYSTEM_MODE_FSIZE_1920x1080 | FAPEX_VIENC_SYSTEM_MODE_PROGRESSIVE | FAPEX_VIENC_SYSTEM_MODE_HDMI_FREQ60)
#define FAPEX_VIENC_SYSTEM_MODE_HDMI_1360x768x60P   (FAPEX_VIENC_SYSTEM_MODE_FSIZE_1360x768  | FAPEX_VIENC_SYSTEM_MODE_PROGRESSIVE | FAPEX_VIENC_SYSTEM_MODE_HDMI_FREQ60)
#define FAPEX_VIENC_SYSTEM_MODE_HDMI_1280x720x24P   (FAPEX_VIENC_SYSTEM_MODE_FSIZE_1280x720  | FAPEX_VIENC_SYSTEM_MODE_PROGRESSIVE | FAPEX_VIENC_SYSTEM_MODE_HDMI_FREQ24)
#define FAPEX_VIENC_SYSTEM_MODE_HDMI_1280x720x25P   (FAPEX_VIENC_SYSTEM_MODE_FSIZE_1280x720  | FAPEX_VIENC_SYSTEM_MODE_PROGRESSIVE | FAPEX_VIENC_SYSTEM_MODE_HDMI_FREQ25)
#define FAPEX_VIENC_SYSTEM_MODE_HDMI_1280x720x30P   (FAPEX_VIENC_SYSTEM_MODE_FSIZE_1280x720  | FAPEX_VIENC_SYSTEM_MODE_PROGRESSIVE | FAPEX_VIENC_SYSTEM_MODE_HDMI_FREQ30)
#define FAPEX_VIENC_SYSTEM_MODE_HDMI_1280x720x50P   (FAPEX_VIENC_SYSTEM_MODE_FSIZE_1280x720  | FAPEX_VIENC_SYSTEM_MODE_PROGRESSIVE | FAPEX_VIENC_SYSTEM_MODE_HDMI_FREQ50)
#define FAPEX_VIENC_SYSTEM_MODE_HDMI_1280x720x60P   (FAPEX_VIENC_SYSTEM_MODE_FSIZE_1280x720  | FAPEX_VIENC_SYSTEM_MODE_PROGRESSIVE | FAPEX_VIENC_SYSTEM_MODE_HDMI_FREQ60)
#define FAPEX_VIENC_SYSTEM_MODE_HDMI_1024x768x60P   (FAPEX_VIENC_SYSTEM_MODE_FSIZE_1024x768  | FAPEX_VIENC_SYSTEM_MODE_PROGRESSIVE | FAPEX_VIENC_SYSTEM_MODE_HDMI_FREQ60)
#define FAPEX_VIENC_SYSTEM_MODE_HDMI_800x600x60P    (FAPEX_VIENC_SYSTEM_MODE_FSIZE_800x600   | FAPEX_VIENC_SYSTEM_MODE_PROGRESSIVE | FAPEX_VIENC_SYSTEM_MODE_HDMI_FREQ60)
#define FAPEX_VIENC_SYSTEM_MODE_HDMI_720x576x50P    (FAPEX_VIENC_SYSTEM_MODE_FSIZE_720x576   | FAPEX_VIENC_SYSTEM_MODE_PROGRESSIVE | FAPEX_VIENC_SYSTEM_MODE_HDMI_FREQ50)
#define FAPEX_VIENC_SYSTEM_MODE_HDMI_720x576x50I    (FAPEX_VIENC_SYSTEM_MODE_FSIZE_720x576   | FAPEX_VIENC_SYSTEM_MODE_INTERLACE   | FAPEX_VIENC_SYSTEM_MODE_HDMI_FREQ50)
#define FAPEX_VIENC_SYSTEM_MODE_HDMI_720x480x60P    (FAPEX_VIENC_SYSTEM_MODE_FSIZE_720x480   | FAPEX_VIENC_SYSTEM_MODE_PROGRESSIVE | FAPEX_VIENC_SYSTEM_MODE_HDMI_FREQ60)
#define FAPEX_VIENC_SYSTEM_MODE_HDMI_720x480x60I    (FAPEX_VIENC_SYSTEM_MODE_FSIZE_720x480   | FAPEX_VIENC_SYSTEM_MODE_INTERLACE   | FAPEX_VIENC_SYSTEM_MODE_HDMI_FREQ60)
#define FAPEX_VIENC_SYSTEM_MODE_HDMI_640x480x60P    (FAPEX_VIENC_SYSTEM_MODE_FSIZE_640x480   | FAPEX_VIENC_SYSTEM_MODE_PROGRESSIVE | FAPEX_VIENC_SYSTEM_MODE_HDMI_FREQ60)

#define FAPEX_VIENC_SYSTEM_MODE_UNKNOWN             (-1)        /*!< unknown system */
#define FAPEX_VIENC_SYSTEM_MODE_1920x1080x2397P     (FAPEX_VIENC_SYSTEM_MODE_HDMI_1920x1080x24P | FAPEX_VIENC_SYSTEM_MODE_FREQ2397)
#define FAPEX_VIENC_SYSTEM_MODE_1920x1080x24P       (FAPEX_VIENC_SYSTEM_MODE_HDMI_1920x1080x24P | FAPEX_VIENC_SYSTEM_MODE_FREQ2400)
#define FAPEX_VIENC_SYSTEM_MODE_1920x1080x25P       (FAPEX_VIENC_SYSTEM_MODE_HDMI_1920x1080x25P | FAPEX_VIENC_SYSTEM_MODE_FREQ2500)
#define FAPEX_VIENC_SYSTEM_MODE_1920x1080x2997P     (FAPEX_VIENC_SYSTEM_MODE_HDMI_1920x1080x30P | FAPEX_VIENC_SYSTEM_MODE_FREQ2997)
#define FAPEX_VIENC_SYSTEM_MODE_1920x1080x30P       (FAPEX_VIENC_SYSTEM_MODE_HDMI_1920x1080x30P | FAPEX_VIENC_SYSTEM_MODE_FREQ3000)
#define FAPEX_VIENC_SYSTEM_MODE_1920x1080x50P       (FAPEX_VIENC_SYSTEM_MODE_HDMI_1920x1080x50P | FAPEX_VIENC_SYSTEM_MODE_FREQ5000)
#define FAPEX_VIENC_SYSTEM_MODE_1920x1080x5994P     (FAPEX_VIENC_SYSTEM_MODE_HDMI_1920x1080x60P | FAPEX_VIENC_SYSTEM_MODE_FREQ5994)
#define FAPEX_VIENC_SYSTEM_MODE_1920x1080x60P       (FAPEX_VIENC_SYSTEM_MODE_HDMI_1920x1080x60P | FAPEX_VIENC_SYSTEM_MODE_FREQ6000)
#define FAPEX_VIENC_SYSTEM_MODE_1920x1080x50I       (FAPEX_VIENC_SYSTEM_MODE_HDMI_1920x1080x50I | FAPEX_VIENC_SYSTEM_MODE_FREQ5000)
#define FAPEX_VIENC_SYSTEM_MODE_1920x1080x5994I     (FAPEX_VIENC_SYSTEM_MODE_HDMI_1920x1080x60I | FAPEX_VIENC_SYSTEM_MODE_FREQ5994)
#define FAPEX_VIENC_SYSTEM_MODE_1920x1080x60I       (FAPEX_VIENC_SYSTEM_MODE_HDMI_1920x1080x60I | FAPEX_VIENC_SYSTEM_MODE_FREQ6000)
#define FAPEX_VIENC_SYSTEM_MODE_1360x768x5994P      (FAPEX_VIENC_SYSTEM_MODE_HDMI_1360x768x60P  | FAPEX_VIENC_SYSTEM_MODE_FREQ5994)
#define FAPEX_VIENC_SYSTEM_MODE_1280x720x2397P      (FAPEX_VIENC_SYSTEM_MODE_HDMI_1280x720x24P  | FAPEX_VIENC_SYSTEM_MODE_FREQ2397)
#define FAPEX_VIENC_SYSTEM_MODE_1280x720x24P        (FAPEX_VIENC_SYSTEM_MODE_HDMI_1280x720x24P  | FAPEX_VIENC_SYSTEM_MODE_FREQ2400)
#define FAPEX_VIENC_SYSTEM_MODE_1280x720x25P        (FAPEX_VIENC_SYSTEM_MODE_HDMI_1280x720x25P  | FAPEX_VIENC_SYSTEM_MODE_FREQ2500)
#define FAPEX_VIENC_SYSTEM_MODE_1280x720x2997P      (FAPEX_VIENC_SYSTEM_MODE_HDMI_1280x720x30P  | FAPEX_VIENC_SYSTEM_MODE_FREQ2997)
#define FAPEX_VIENC_SYSTEM_MODE_1280x720x30P        (FAPEX_VIENC_SYSTEM_MODE_HDMI_1280x720x30P  | FAPEX_VIENC_SYSTEM_MODE_FREQ3000)
#define FAPEX_VIENC_SYSTEM_MODE_1280x720x50P        (FAPEX_VIENC_SYSTEM_MODE_HDMI_1280x720x50P  | FAPEX_VIENC_SYSTEM_MODE_FREQ5000)
#define FAPEX_VIENC_SYSTEM_MODE_1280x720x5994P      (FAPEX_VIENC_SYSTEM_MODE_HDMI_1280x720x60P  | FAPEX_VIENC_SYSTEM_MODE_FREQ5994)
#define FAPEX_VIENC_SYSTEM_MODE_1280x720x60P        (FAPEX_VIENC_SYSTEM_MODE_HDMI_1280x720x60P  | FAPEX_VIENC_SYSTEM_MODE_FREQ6000)
#define FAPEX_VIENC_SYSTEM_MODE_1024x768x5994P      (FAPEX_VIENC_SYSTEM_MODE_HDMI_1024x768x60P  | FAPEX_VIENC_SYSTEM_MODE_FREQ5994)
#define FAPEX_VIENC_SYSTEM_MODE_800x600x5994P       (FAPEX_VIENC_SYSTEM_MODE_HDMI_800x600x60P   | FAPEX_VIENC_SYSTEM_MODE_FREQ5994)
#define FAPEX_VIENC_SYSTEM_MODE_720x576x50I         (FAPEX_VIENC_SYSTEM_MODE_HDMI_720x576x50I   | FAPEX_VIENC_SYSTEM_MODE_FREQ5000)
#define FAPEX_VIENC_SYSTEM_MODE_720x576x50P         (FAPEX_VIENC_SYSTEM_MODE_HDMI_720x576x50P   | FAPEX_VIENC_SYSTEM_MODE_FREQ5000)
#define FAPEX_VIENC_SYSTEM_MODE_720x480x5994P       (FAPEX_VIENC_SYSTEM_MODE_HDMI_720x480x60P   | FAPEX_VIENC_SYSTEM_MODE_FREQ5994)
#define FAPEX_VIENC_SYSTEM_MODE_720x480x5994I       (FAPEX_VIENC_SYSTEM_MODE_HDMI_720x480x60I   | FAPEX_VIENC_SYSTEM_MODE_FREQ5994)
#define FAPEX_VIENC_SYSTEM_MODE_640x480x5994P       (FAPEX_VIENC_SYSTEM_MODE_HDMI_640x480x60P   | FAPEX_VIENC_SYSTEM_MODE_FREQ5994)
#define FAPEX_VIENC_SYSTEM_MODE_LCD_640x480x50P     (FAPEX_VIENC_SYSTEM_MODE_LCD | FAPEX_VIENC_SYSTEM_MODE_FSIZE_1024x768 | FAPEX_VIENC_SYSTEM_MODE_PROGRESSIVE | FAPEX_VIENC_SYSTEM_MODE_FREQ5000)
#define FAPEX_VIENC_SYSTEM_MODE_LCD_1024x768x50P    (FAPEX_VIENC_SYSTEM_MODE_LCD | FAPEX_VIENC_SYSTEM_MODE_FSIZE_640x480  | FAPEX_VIENC_SYSTEM_MODE_PROGRESSIVE | FAPEX_VIENC_SYSTEM_MODE_FREQ5000)

/*
*************************************************************************
*************************************************************************
** Data types
*************************************************************************
*************************************************************************/

/*!
*************************************************************************
** \brief open parameters for the VIENC driver
**
** The related ioctl command type :
** \ref vienc_cmd_open_subid "FAPEX_IOCCOM_VIENC_OPEN_SUBID"
**
** \sa FAPEX_VIENC_Ioctl
*************************************************************************/
typedef struct {
    /*!
        Video Encoder device sub_id<br>
        \param[in]  sub_id  Device minor_no (#FAPEXDEV_VIENC_SUB_MINOR_FIRST .. #FAPEXDEV_VIENC_SUB_MINOR_LAST).
        \param[out] sub_id  Device identifier for VIENC channel.
    */
    int32_t  sub_id;
    /*!
        encoder type<br>
        \param #FAPEX_VIENC_ENCODER_UNKNOWN Specifies an unknown output encoder Type
        \param #FAPEX_VIENC_ENCODER_LCD     Specifies an LCD output encoder Type
        \param #FAPEX_VIENC_ENCODER_SD      Specifies a SD output encoder Type
        \param #FAPEX_VIENC_ENCODER_HD      Specifies a HD output encoder Type
    */
    int32_t  encoderType;
    /*!
        interlace progress type<br>
        \param #FAPEX_VIENC_IP_MODE_UNKNOWN          Specifies an unknown interlace or progressive encoder Type
        \param #FAPEX_VIENC_IP_MODE_PAL              PAL Interlace mode available on VE SD
        \param #FAPEX_VIENC_IP_MODE_NTSC             NTSC Interlace mode available on VE SD
        \param #FAPEX_VIENC_IP_MODE_SECAM            SECAM Interlace mode available on VE SD
        \param #FAPEX_VIENC_IP_MODE_1920x1080x2397P  23.97 Hz Interlace mode available on VE HD,<br>
                                                     Active video region is 1920 pixels x 1080 lines.
        \param #FAPEX_VIENC_IP_MODE_1920x1080x24P    24 Hz Interlace mode available on VE HD,<br>
                                                     Active video region is 1920 pixels x 1080 lines.
        \param #FAPEX_VIENC_IP_MODE_1920x1080x25P    25 Hz Progressive mode available on VE HD,<br>
                                                     Active video region is 1920 pixels x 1080 lines.
        \param #FAPEX_VIENC_IP_MODE_1920x1080x2997P  29.97 Hz Interlace mode available on VE HD,<br>
                                                     Active video region is 1920 pixels x 1080 lines.
        \param #FAPEX_VIENC_IP_MODE_1920x1080x30P    30 Hz Interlace mode available on VE HD,<br>
                                                     Active video region is 1920 pixels x 1080 lines.
        \param #FAPEX_VIENC_IP_MODE_1920x1080x50I    50 Hz Interlace mode available on VE HD,<br>
                                                     Active video region is 1920 pixels x 540 lines.
        \param #FAPEX_VIENC_IP_MODE_1920x1080x5994I  59.94 Hz Interlace mode available on VE HD,<br>
                                                     Active video region is 1920 pixels x 540 lines.
        \param #FAPEX_VIENC_IP_MODE_1920x1080x60I    60 Hz Interlace mode available on VE HD,<br>
                                                     Active video region is 1920 pixels x 540 lines.
        \param #FAPEX_VIENC_IP_MODE_1280x720x2397P   23.97 Hz Progressive mode available on VE HD,<br>
                                                     Active video region is 1280 pixels x 720 lines.
        \param #FAPEX_VIENC_IP_MODE_1280x720x24P     24 Hz Progressive mode available on VE HD,<br>
                                                     Active video region is 1280 pixels x 720 lines.
        \param #FAPEX_VIENC_IP_MODE_1280x720x25P     25 Hz Progressive mode available on VE HD,<br>
                                                     Active video region is 1280 pixels x 720 lines.
        \param #FAPEX_VIENC_IP_MODE_1280x720x2997P   29.97 Hz Progressive mode available on VE HD,<br>
                                                     Active video region is 1280 pixels x 720 lines.
        \param #FAPEX_VIENC_IP_MODE_1280x720x30P     30 Hz Progressive mode available on VE HD,<br>
                                                     Active video region is 1280 pixels x 720 lines.
        \param #FAPEX_VIENC_IP_MODE_1280x720x50P     50 Hz Progressive mode available on VE HD,<br>
                                                     Active video region is 1280 pixels x 720 lines.
        \param #FAPEX_VIENC_IP_MODE_1280x720x5994P   59.94 Hz Progressive mode available on VE HD,<br>
                                                     Active video region is 1280 pixels x 720 lines.
        \param #FAPEX_VIENC_IP_MODE_1280x720x60P     60 Hz Progressive mode available on VE HD,<br>
                                                     Active video region is 1280 pixels x 720 lines.
        \param #FAPEX_VIENC_IP_MODE_720x576x50I      50 Hz Interlace mode available on VE HD,<br>
                                                     Active video region is 1440 pixels x 288 lines.
        \param #FAPEX_VIENC_IP_MODE_720x576x50P      50 Hz Progressive mode available on VE HD,<br>
                                                     Active video region is 720 pixels x 576 lines.
        \param #FAPEX_VIENC_IP_MODE_720x480x5994I    59.94 Hz Interlace mode available on VE HD,<br>
                                                     Active video region is 1440 pixels x 240 lines.
        \param #FAPEX_VIENC_IP_MODE_720x480x5994P    59.94 Hz Progressive mode available on VE HD,<br>
                                                     Active video region is 720 pixels x 480 lines.
        \param #FAPEX_VIENC_IP_MODE_640x480x5994P    59.94 Hz Progressive mode available on VE HD,<br>
                                                     Active video region is 1280 pixels x 960 lines.
        \param #FAPEX_VIENC_IP_MODE_800x600x5994P    59.94 Hz Progressive mode available on VE HD,<br>
                                                     Active video region is 1600 pixels x 1200 lines.
        \param #FAPEX_VIENC_IP_MODE_1024x768x5994P   59.94 Hz Progressive mode available on VE HD,<br>
                                                     Active video region is 2048 pixels x 1536 lines.
        \param #FAPEX_VIENC_IP_MODE_1360x768x5994P   59.94 Hz Progressive mode available on VE HD,<br>
                                                     Active video region is 2720 pixels x 1536 lines.
        \param #FAPEX_VIENC_IP_MODE_LCD_640x480x50P  LCD progressive display available on GPIOs only,<br>
                                                     The frame rate is same that of VE HD
        \param #FAPEX_VIENC_IP_MODE_LCD_1024x768x50P LCD progressive display available on GPIOs only,<br>
                                                     The frame rate is same that of VE HD
    */
    int32_t  interlaceProgressive;
    /*!
        colour system type<br>
        \param #FAPEX_VIENC_COLOUR_SYSTEM_UNKNOWN    Specifies an unknown colour system encoder Type
        \param #FAPEX_VIENC_COLOUR_SYSTEM_SECAM      Specifies \b SECAM system colour.
        \param #FAPEX_VIENC_COLOUR_SYSTEM_NTSC       Specifies \b NTSC system colour.
        \param #FAPEX_VIENC_COLOUR_SYSTEM_NTSC_4_43  Specifies \b NTSC_4_43 system colour.
        \param #FAPEX_VIENC_COLOUR_SYSTEM_PAL        Specifies \b PAL system colour.
        \param #FAPEX_VIENC_COLOUR_SYSTEM_PAL_60     Specifies \b PAL60 system colour.
        \param #FAPEX_VIENC_COLOUR_SYSTEM_PAL_M      Specifies \b PAL_M system colour.
        \param #FAPEX_VIENC_COLOUR_SYSTEM_PAL_N      Specifies \b PAL_N system colour.
                                                     This is PAL-N except for Argentinia.<br>
                                                     It uses the same colour carrier freq as PAL-BGHI
                                                     but the pedestal is different.
        \param #FAPEX_VIENC_COLOUR_SYSTEM_PAL_N_ARG  Specifies \b PAL_N_ARG system colour.
                                                     This is PAL-N used only in Argentinia.<br>
                                                     It has a different colour carrier freq than PAL-BGHI.
    */
    int32_t  colourSystem;
    /*!
        colour component type<br>
        \param #FAPEX_VIENC_CCOMPONENT_UNKNOWN         Specifies an unknown colour component
        \param #FAPEX_VIENC_CCOMPONENT_DAC_POWER_DOWN  Powers down the DACs of VE SD or VE HD
        \param #FAPEX_VIENC_CCOMPONENT_DIGITAL_RGB     Specifies Digital RGB output on VE HD ONLY
        \param #FAPEX_VIENC_CCOMPONENT_ANALOG_RGB      Specifies Analog RGB output on VE HD ONLY
        \param #FAPEX_VIENC_CCOMPONENT_RGB             Specifies Analog & Digital RGB output on VE HD ONLY 
        \param #FAPEX_VIENC_CCOMPONENT_DIGITAL_YPrPb   Specifies Digital YPrPb output on VE HD ONLY 
        \param #FAPEX_VIENC_CCOMPONENT_ANALOG_YPrPb    Specifies YPrPb output on VE HD ONLY
        \param #FAPEX_VIENC_CCOMPONENT_YPrPb           Specifies Analog & Digital YPrPb output on VE HD ONLY 
        \param #FAPEX_VIENC_CCOMPONENT_COLOURBAR       Specifies a standard \b COLOUR \b BAR. on VE SD only
        \param #FAPEX_VIENC_CCOMPONENT_CVBS            Specifies only CVBS on VE SD ONLY
        \param #FAPEX_VIENC_CCOMPONENT_YC              Specifies only CVBS on VE SD ONLY 
        \param #FAPEX_VIENC_CCOMPONENT_CVBS_YC         Specifies CVBS with YC output on VE SD ONLY for S Video
    */
    int32_t  colourComponent;
    /*!
        Cross-chrominance filter switch<br>
        This parameter is used only for VE SD.<br>
        \param #FAPEX_VIENC_CROSS_CHROMIANACE_FILTER_UNKNOWN  Specifies an unknown Cross chrominance filter state
        \param #FAPEX_VIENC_CROSS_CHROMIANACE_FILTER_OFF      Parameter to turn off Cross chrominance filter
        \param #FAPEX_VIENC_CROSS_CHROMIANACE_FILTER_ON       Parameter to turn on Cross chrominance filter
    */
    int32_t  crossChrominanceFilter;

} FAPEX_VIENC_OpenSubIDParamStrT;

// encoder type
#define FAPEX_VIENC_ENCODER_UNKNOWN (int32_t)-1 /*!< Specifies an unknown output encoder Type */
#define FAPEX_VIENC_ENCODER_LCD     0           /*!< Specifies an LCD output encoder Type */
#define FAPEX_VIENC_ENCODER_SD      1           /*!< Specifies a SD output encoder Type */
#define FAPEX_VIENC_ENCODER_HD      2           /*!< Specifies a HD output encoder Type */

// interlace progress type
#define FAPEX_VIENC_IP_MODE_UNKNOWN          0   /*!< Specifies an unknown interlace or progressive encoder Type*/
#define FAPEX_VIENC_IP_MODE_PAL              1   /*!< PAL Interlace mode available on VE SD*/
#define FAPEX_VIENC_IP_MODE_NTSC             2   /*!< NTSC Interlace mode available on VE SD*/
#define FAPEX_VIENC_IP_MODE_SECAM            3   /*!< SECAM Interlace mode available on VE SD*/
#define FAPEX_VIENC_IP_MODE_1920x1080x2397P  4   /*!< 23.97 Hz Interlace mode available on VE HD,<br>
                                                      Active video region is 1920 pixels x 1080 lines.*/
#define FAPEX_VIENC_IP_MODE_1920x1080x24P    5   /*!< 24 Hz Interlace mode available on VE HD,<br>
                                                      Active video region is 1920 pixels x 1080 lines.*/
#define FAPEX_VIENC_IP_MODE_1920x1080x25P    6   /*!< 25 Hz Progressive mode available on VE HD,<br>
                                                      Active video region is 1920 pixels x 1080 lines.*/
#define FAPEX_VIENC_IP_MODE_1920x1080x2997P  7   /*!< 29.97 Hz Interlace mode available on VE HD,<br>
                                                      Active video region is 1920 pixels x 1080 lines.*/
#define FAPEX_VIENC_IP_MODE_1920x1080x30P    8   /*!< 30 Hz Interlace mode available on VE HD,<br>
                                                      Active video region is 1920 pixels x 1080 lines.*/
#define FAPEX_VIENC_IP_MODE_1920x1080x50I    9   /*!< 50 Hz Interlace mode available on VE HD,<br>
                                                      Active video region is 1920 pixels x 540 lines.*/
#define FAPEX_VIENC_IP_MODE_1920x1080x50P    10  /*!< 50 Hz Progressive mode available on VE HD,<br>
                                                      Active video region is 1920 pixels x 540 lines.*/
#define FAPEX_VIENC_IP_MODE_1920x1080x5994I  11  /*!< 59.94 Hz Interlace mode available on VE HD,<br>
                                                      Active video region is 1920 pixels x 540 lines.*/
#define FAPEX_VIENC_IP_MODE_1920x1080x5994P  12  /*!< 59.94 Hz Progressive mode available on VE HD,<br>
                                                      Active video region is 1920 pixels x 540 lines.*/
#define FAPEX_VIENC_IP_MODE_1920x1080x60I    13  /*!< 60 Hz Interlace mode available on VE HD,<br>
                                                      Active video region is 1920 pixels x 540 lines.*/
#define FAPEX_VIENC_IP_MODE_1920x1080x60P    14  /*!< 60 Hz Progressive mode available on VE HD,<br>
                                                      Active video region is 1920 pixels x 540 lines.*/
#define FAPEX_VIENC_IP_MODE_1280x720x2397P   15  /*!< 23.97 Hz Progressive mode available on VE HD,<br>
                                                      Active video region is 1280 pixels x 720 lines.*/
#define FAPEX_VIENC_IP_MODE_1280x720x24P     16  /*!< 24 Hz Progressive mode available on VE HD,<br>
                                                      Active video region is 1280 pixels x 720 lines.*/
#define FAPEX_VIENC_IP_MODE_1280x720x25P     17  /*!< 25 Hz Progressive mode available on VE HD,<br>
                                                      Active video region is 1280 pixels x 720 lines.*/
#define FAPEX_VIENC_IP_MODE_1280x720x2997P   18  /*!< 29.97 Hz Progressive mode available on VE HD,<br>
                                                      Active video region is 1280 pixels x 720 lines.*/
#define FAPEX_VIENC_IP_MODE_1280x720x30P     19  /*!< 30 Hz Progressive mode available on VE HD,<br>
                                                      Active video region is 1280 pixels x 720 lines.*/
#define FAPEX_VIENC_IP_MODE_1280x720x50P     20  /*!< 50 Hz Progressive mode available on VE HD,<br>
                                                      Active video region is 1280 pixels x 720 lines.*/
#define FAPEX_VIENC_IP_MODE_1280x720x5994P   21  /*!< 59.94 Hz Progressive mode available on VE HD,<br>
                                                      Active video region is 1280 pixels x 720 lines.*/
#define FAPEX_VIENC_IP_MODE_1280x720x60P     22  /*!< 60 Hz Progressive mode available on VE HD,<br>
                                                      Active video region is 1280 pixels x 720 lines.*/
#define FAPEX_VIENC_IP_MODE_720x576x50I      23  /*!< 50 Hz Interlace mode available on VE HD,<br>
                                                      Active video region is 1440 pixels x 288 lines.*/
#define FAPEX_VIENC_IP_MODE_720x576x50P      24  /*!< 50 Hz Progressive mode available on VE HD,<br>
                                                      Active video region is 720 pixels x 576 lines.*/
#define FAPEX_VIENC_IP_MODE_720x480x5994I    25  /*!< 59.94 Hz Interlace mode available on VE HD,<br>
                                                      Active video region is 1440 pixels x 240 lines.*/
#define FAPEX_VIENC_IP_MODE_720x480x5994P    26  /*!< 59.94 Hz Progressive mode available on VE HD,<br>
                                                      Active video region is 720 pixels x 480 lines.*/
#define FAPEX_VIENC_IP_MODE_640x480x5994P    27  /*!< 59.94 Hz Progressive mode available on VE HD
                                                      Active video region is 1280 pixels x 960 lines.*/
#define FAPEX_VIENC_IP_MODE_800x600x5994P    28  /*!< 59.94 Hz Progressive mode available on VE HD
                                                      Active video region is 1600 pixels x 1200 lines.*/
#define FAPEX_VIENC_IP_MODE_1024x768x5994P   29  /*!< 59.94 Hz Progressive mode available on VE HD
                                                      Active video region is 2048 pixels x 1536 lines.*/
#define FAPEX_VIENC_IP_MODE_1360x768x5994P   30  /*!< 59.94 Hz Progressive mode available on VE HD
                                                      Active video region is 2720 pixels x 1536 lines.*/
#define FAPEX_VIENC_IP_MODE_LCD_640x480x50P  31  /*!< LCD progressive display available on GPIOs only,<br>
                                                      The frame rate is same that of VE HD */
#define FAPEX_VIENC_IP_MODE_LCD_1024x768x50P 32  /*!< LCD progressive display available on GPIOs only,<br>
                                                      The frame rate is same that of VE HD */

// colour system type
#define FAPEX_VIENC_COLOUR_SYSTEM_UNKNOWN  (-1) /*!< Specifies an unknown colour system encoder Type */
#define FAPEX_VIENC_COLOUR_SYSTEM_SECAM      0  /*!< Specifies \b SECAM system colour. */
#define FAPEX_VIENC_COLOUR_SYSTEM_NTSC       1  /*!< Specifies \b NTSC system colour. */
#define FAPEX_VIENC_COLOUR_SYSTEM_NTSC_4_43  2  /*!< Specifies \b NTSC_4_43 system colour. */
#define FAPEX_VIENC_COLOUR_SYSTEM_PAL        3  /*!< Specifies \b PAL system colour. */
#define FAPEX_VIENC_COLOUR_SYSTEM_PAL_60     4  /*!< Specifies \b PAL60 system colour. */
#define FAPEX_VIENC_COLOUR_SYSTEM_PAL_M      5  /*!< Specifies \b PAL_M system colour. */
#define FAPEX_VIENC_COLOUR_SYSTEM_PAL_N      6  /*!< Specifies \b PAL_N system colour.
                                                     This is PAL-N except for Argentinia.<br>
                                                     It uses the same colour carrier freq as PAL-BGHI
                                                     but the pedestal is different. */
#define FAPEX_VIENC_COLOUR_SYSTEM_PAL_N_ARG  7  /*!< Specifies \b PAL_N_ARG system colour.
                                                     This is PAL-N used only in Argentinia.<br>
                                                     It has a different colour carrier freq than PAL-BGHI. */

// colour component type
#define FAPEX_VIENC_CCOMPONENT_UNKNOWN        (-1)  /*!< Specifies an unknown colour component */
#define FAPEX_VIENC_CCOMPONENT_DAC_POWER_DOWN   0   /*!< Powers down the DACs of VE SD or VE HD */
#define FAPEX_VIENC_CCOMPONENT_DIGITAL_RGB      1   /*!< Specifies Digital RGB output on VE HD ONLY */
#define FAPEX_VIENC_CCOMPONENT_ANALOG_RGB       2   /*!< Specifies Analog RGB output on VE HD ONLY */
#define FAPEX_VIENC_CCOMPONENT_RGB              3   /*!< Specifies Analog & Digital RGB output on VE HD ONLY */
#define FAPEX_VIENC_CCOMPONENT_DIGITAL_YPrPb    4   /*!< Specifies Digital YPrPb output on VE HD ONLY */
#define FAPEX_VIENC_CCOMPONENT_ANALOG_YPrPb     5   /*!< Specifies YPrPb output on VE HD ONLY */
#define FAPEX_VIENC_CCOMPONENT_YPrPb            6   /*!< Specifies Analog & Digital YPrPb output on VE HD ONLY */
#define FAPEX_VIENC_CCOMPONENT_COLOURBAR        7   /*!< Specifies a standard \b COLOUR \b BAR. on VE SD only */
#define FAPEX_VIENC_CCOMPONENT_CVBS             8   /*!< Specifies only CVBS on VE SD ONLY */
#define FAPEX_VIENC_CCOMPONENT_YC               9   /*!< Specifies only CVBS on VE SD ONLY */
#define FAPEX_VIENC_CCOMPONENT_CVBS_YC         10   /*!< Specifies CVBS with YC output on VE SD ONLY for S Video */

// Cross-chrominance filter switch
#define FAPEX_VIENC_CROSS_CHROMIANACE_FILTER_UNKNOWN  (-1)  /*!< Specifies an unknown Cross chrominance filter state */
#define FAPEX_VIENC_CROSS_CHROMIANACE_FILTER_OFF        0   /*!< Parameter to turn off Cross chrominance filter */
#define FAPEX_VIENC_CROSS_CHROMIANACE_FILTER_ON         1   /*!< Parameter to turn on Cross chrominance filter */

// DAC assignment
#define FAPEX_VIENC_DAC_ASSIGNMENT_012_HD_345_SD       0 /*!<Specifies DACs 012 for HD and DACs345 for SD analog encoder */
#define FAPEX_VIENC_DAC_ASSIGNMENT_012_SD_345_HD       1 /*!<Specifies DACs 012 for SD and DACs345 for HD analog encoder */

/*!
*************************************************************************
** \brief channel status parameters for the VIENC driver
**
** The related ioctl command type :
** \ref vienc_cmd_get_status "FAPEX_IOCCOM_VIENC_GET_STATUS"
**
** \sa FAPEX_VIENC_Ioctl
*************************************************************************/
typedef struct {
    /*!
        Video Encoder status<br>
        \param #FAPEX_STATUS_OPENED   opened status
        \param #FAPEX_STATUS_READY    ready starts
        \param #FAPEX_STATUS_RUNNING  running status
    */
    int32_t     status;

} FAPEX_VIENC_StatusStrT;


/*!
*************************************************************************
** \brief ioctl specific command for the VIENC driver
*************************************************************************/
#define FAPEX_IOCCOM_VIENC_OPEN         _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIENC + 0,   int32_t)
#define FAPEX_IOCCOM_VIENC_OPEN_SUBID   _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_SUBID_VIENC, FAPEX_VIENC_OpenSubIDParamStrT)
#define FAPEX_IOCCOM_VIENC_GET_STATUS   _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIENC + 1,   FAPEX_VIENC_StatusStrT)
#define FAPEX_IOCCOM_VIENC_GET_WSSDATA  _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIENC + 2,   uint32_t)
#define FAPEX_IOCCOM_VIENC_SET_WSSDATA  _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIENC + 2,   uint32_t)

/* Martin added at 20100513 for SMARTGO required*/
#define FAPEX_IOCCOM_VIENC_SET_CCOMPONENT   _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIENC + 3, uint32_t)
#define FAPEX_IOCCOM_VIENC_GET_CCOMPONENT   _IOR(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIENC + 3, uint32_t) 
#define FAPEX_IOCCOM_VIENC_ENABLE_WSS    _IO(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIENC + 4)
#define FAPEX_IOCCOM_VIENC_DISABLE_WSS    _IO(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIENC + 5)
/* Martin added end */
/* for H61 DAC assignment */
#define FAPEX_IOCCOM_VIENC_SET_DAC       _IOW(FAPEX_DEV_TYPE, FAPEX_IOCCOM_NB_VIENC+6, uint32_t) 


/*
*************************************************************************
*************************************************************************
** prototype for application
*************************************************************************
*************************************************************************/
#ifndef __LINUX__
#include "fapex/abst_types.h"

#ifdef __cplusplus
extern "C" {
#endif

FAPEXOPENRET_T FAPEX_VIENC_Open(FAPEXOPENARG1_T id, FAPEXOPENARG2_T flags, FAPEXOPENARG3_T argptr);
FAPEXRELRET_T  FAPEX_VIENC_Close(FAPEXRELARG1_T id);
FAPEXIOCRET_T  FAPEX_VIENC_Ioctl(FAPEXIOCARG1_T id, FAPEXIOCARG2_T cmd, FAPEXIOCARG3_T pArg);

#ifdef __cplusplus
}
#endif
#endif /* !__LINUX__ */

#endif /* !_FAPEX_ABST_LIBVIENC_H */

