/*!
*******************************************************************************
**
** \file        ./applications/fastboot/src/fastboot.c
** 
** \version     $Id$
**
** \author      
**
** \brief       Bootloader library of 'fastboot'
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU MICROELECTRONICS
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2008-2010 by Fujitsu Microelectronics Europe GmbH
** (C) Copyright 2009-2010 by Fujitsu Microelectronics Limited
*******************************************************************************
*/

/*---------------------------------------------------------------------------*/
/* include files                                                             */
/*---------------------------------------------------------------------------*/

#include <fapi/sys_services.h>
#include <fapi/drv_flash.h>
#include <rtos/rtos.h>
#include <fapi/drv_dma.h>
#include "fastboot.h"
#include "drv_flash_priv.h"
#if (FBOOT_ENABLE_CRC_CHECK)
    #include "fastboot_crc.h"
#endif /* FBOOT_ENABLE_CRC_CHECK */

/*!
*******************************************************************************
** 
** \brief  Transfer Image by PIO.
** 
** \param  pHandle       Fast Bootloader handle.
** \param  pSrcAddr      Image source address
** \param  pDestAddr     Destination address
** \param  nSize         data size to be transfered
** 
** \return 
** - #FAPI_OK  if transration completed.
** - #FAPI_ERR_BAD_PARAMETER if a given parameter was invalid.
** - #FAPI_ERR_DEVICE_BUSY if PIO transmission failed.
******************************************************************************/
int32_t FBOOT_ExecutePioTransmission (FBOOT_HandleT* pHandle, void*pSrcAddr, void* pDstAddr, 
                                        uint32_t nSize)
{
    int32_t retVal;
    
    /* Check parameter */
    if( (pHandle==NULL) || (pSrcAddr==NULL) || (pDstAddr==NULL)){
        /*lint -save -e534 */
        FAPI_SYS_PRINT_MSG("ExecutePioTransmission: Bad parameter.\r\n");
        /*lint -restore */
        return FAPI_ERR_BAD_PARAMETER;
    }

    //FAPI_SYS_PRINT_MSG("FAPI_FLASH_ReadBytes-- src: 0x%x size: 0x%x\n", pSrcAddr, nSize);

    /* Transrate Image */
    retVal = FAPI_FLASH_ReadBytes ( pHandle->hFlashHandle, (uint32_t)pSrcAddr,
                                            nSize, pDstAddr);
    if ( retVal != FAPI_OK ){
        return( FAPI_ERR_DEVICE_BUSY );
    }
    return FAPI_OK;
}


/*!
*******************************************************************************
** 
** \brief  Initialize DMA transmission.
** 
** \param  pHandle       Fast Bootloader handle.
** 
** \return 
** - #FAPI_OK  if transration completed.
** - #FAPI_ERR_BAD_PARAMETER if a given parameter was invalid.
** - #FAPI_ERR_SEMAPHORE_CREATE if semaphore creation was invalid.
** - #FAPI_ERR_DEVICE_BUSY if DMA allocation was failed.
******************************************************************************/
int32_t FBOOT_InitBurstTransmission (FBOOT_HandleT* pHandle)
{
    uint32_t i, selfDestroy=0;
    
    /* Check parameter */
    if( pHandle==NULL){
        /*lint -save -e534 */
        FAPI_SYS_PRINT_MSG("InitBurstTransmission: Bad parameter.\r\n");
        /*lint -restore */

        return FAPI_ERR_BAD_PARAMETER;
    }
    
    /* Create Semaphore */
    pHandle->hSemaphore= FAPI_SYS_CREATE_SEMAPHORE(0);
    if(pHandle->hSemaphore == NULL){
        /*lint -save -e534 */
        FAPI_SYS_PRINT_MSG("Failed to create semaphore.\r\n" );
        /*lint -restore */
        return FAPI_ERR_SEMAPHORE_CREATE;
    }
    
    /* DMA Setup loop */
    for(i=0;i<FBOOT_DMA_NUM;i++){
        /* Allocate DMA */
        pHandle->pDMARequest[i]
            = FAPI_DMA_AllocateRequest( FAPI_DMA_FEATURE_SDRAM_COPY, selfDestroy);
        
        /* Error Case */
        if(pHandle->pDMARequest[i]==NULL){
            (void)FBOOT_DeinitBurstTransmission(pHandle);
            return FAPI_ERR_DEVICE_BUSY;
        }
        /*
         * Setup DMA parameters
         */
        pHandle->pDMARequest[i]->preEnableCallback = 0;
        pHandle->pDMARequest[i]->postEnableCallback = 0;
        
        /* Set DMA Callback function */
        pHandle->pDMARequest[i]->postFinishCallback = (FAPI_DMA_CallbackT)FBOOT_DMA_Callback;
        pHandle->pDMARequest[i]->optHandle = 0;
        pHandle->pDMARequest[i]->optData = (void*)pHandle; 
        pHandle->pDMARequest[i]->optFinishCallback = 0;
        pHandle->pDMARequest[i]->optChangeCallback = 0;
        pHandle->pDMARequest[i]->channelConfig.chConfig = 0;
        pHandle->pDMARequest[i]->channelConfig.chLladdr = 0;

        pHandle->pDMARequest[i]->channelConfig.chRdaddr_PeripheralAddress = 0x0;
        pHandle->pDMARequest[i]->channelConfig.chRdaddr_OffsetAddress = 0;
        pHandle->pDMARequest[i]->channelConfig.chRdline_Lines = 0x3FF;

        pHandle->pDMARequest[i]->channelConfig.chRdinc_LineLength = 0x2;
        pHandle->pDMARequest[i]->channelConfig.chRdinc_LineIncrement = 0x2;
        pHandle->pDMARequest[i]->channelConfig.chRdlpaddr_LoopAddress = 0;
        pHandle->pDMARequest[i]->channelConfig.chRdaddr_EndianSwap = 0;

        pHandle->pDMARequest[i]->channelConfig.chWraddr_PeripheralAddress = 0x0;
        pHandle->pDMARequest[i]->channelConfig.chWraddr_OffsetAddress = 0;
        pHandle->pDMARequest[i]->channelConfig.chWrline_Lines = 0x3FF;
        pHandle->pDMARequest[i]->channelConfig.chWrinc_LineLength = 0xFFF;
        pHandle->pDMARequest[i]->channelConfig.chWrinc_LineIncrement = 0;
        pHandle->pDMARequest[i]->channelConfig.chWrlpaddr_LoopAddress = 0;
        pHandle->pDMARequest[i]->channelConfig.chWraddr_EndianSwap = 0;
    }
    
    
    return FAPI_OK;
}


/*!
*******************************************************************************
** 
** \brief  Release resources for DMA transmission.
** 
** \param  pHandle       Fast Bootloader handle.
** 
** \return 
** - #FAPI_OK  if transration completed.
** - #FAPI_ERR_BAD_PARAMETER if a given parameter was invalid.
******************************************************************************/
int32_t FBOOT_DeinitBurstTransmission (FBOOT_HandleT* pHandle)
{
    int32_t i;
    
    /* Check parameter */
    if( pHandle==NULL){
        /*lint -save -e534 */
        FAPI_SYS_PRINT_MSG("DeinitBurstTransmission: Bad parameter.\r\n");
        /*lint -restore */

        return FAPI_ERR_BAD_PARAMETER;
    }

    /* destroy Semaphore */
    FAPI_SYS_DESTROY_SEMAPHORE( pHandle->hSemaphore);

    /* DMA Release loop */
    for(i=0;i<FBOOT_DMA_NUM;i++){
        /* Release DMA. */
        if(pHandle->pDMARequest[i] != NULL){
            FAPI_DMA_ReleaseRequest(pHandle->pDMARequest[i]);
        }
    }
    return FAPI_OK;
}

/*!
*******************************************************************************
** 
** \brief  Transfer Image by DMA.
** 
** \param  pHandle       Fast Bootloader handle.
** \param  pSrcAddr      Image source address
** \param  pDestAddr     Destination address
** \param  nSize         data size to be transfered
** 
** \return 
** - #  >= 0                 TxID (if transration completed.)
** - #FAPI_ERR_BAD_PARAMETER if a given parameter was invalid.
** - #FAPI_ERR_DEVICE_BUSY if DMA transmission failed.
** - #FAPI_ERR_OUT_OF_MEMORY if run short of DMA transmission block.
******************************************************************************/
int32_t FBOOT_ExecuteBurstTransmission (FBOOT_HandleT* pHandle, void* pSrcAddr, void* pDstAddr, 
                                        uint32_t nSize)
{
    int32_t retVal = FAPI_OK;
    uint32_t nRemainedSize;
    uint32_t nTxID, nDivideNum;
    uint32_t i,nNo;
    void *pAbsFlashAddr;

    /* Check parameter(NULL access) */
    if( (pHandle==NULL) || (pSrcAddr==NULL) || (pDstAddr==NULL)){
        /*lint -save -e534 */
        FAPI_SYS_PRINT_MSG("ExecuteBurstTransmission: Bad parameter.\r\n");
        /*lint -restore */

        return FAPI_ERR_BAD_PARAMETER;
    }

    /* Check for long word alignment. */
    if( (( (uint32_t)pSrcAddr & 0x3 )!=0)  ||  (( (uint32_t)pDstAddr & 0x3 )!=0) 
            || ((nSize % 4) !=0) ) 
    {
        /*lint -save -e534 */
        FAPI_SYS_PRINT_MSG("DMA memcpy, address is not long word aligned\r\n" );
        /*lint -restore */
        return FAPI_ERR_BAD_PARAMETER;
    }
    
    /* Make up(Count up) nTxID for this transmission. */
    if(  pHandle->nTxBlockCount ==0){
        /* nTxID for first transmission (= first DMA Tx Block) is 0. */
        nTxID=0;
    }
    else{
        /* Count up nTxID from formar transmission.
            "pHandle->sDMATxBlock[ pHandle->nTxBlockCount -1].nTxID" is nTxID which
            is formar DMA Tx Block belongs. */
        nTxID = pHandle->sDMATxBlock[ pHandle->nTxBlockCount -1].nTxID +1;
    }
    
    /* Divide data to DTB(DMA Tx Block) block size. */
    nDivideNum = nSize/FBOOT_DMA_SIZE;
    if((nSize%FBOOT_DMA_SIZE) != 0){
        nDivideNum++;
    }
    
    nRemainedSize = nSize;
    for(i=0;i<nDivideNum;i++){
        
        /* If run short of DMA Tx Block, return error. */
        if( pHandle->nTxBlockCount >= FBOOT_DTB_NUM ){
            retVal = FAPI_ERR_OUT_OF_MEMORY;
            /*lint -save -e534 */
            FAPI_SYS_PRINT_MSG("Run short of DMA Tx Block \r\n");
            /*lint -restore */
            retVal=FAPI_ERR_BAD_PARAMETER;
            break;
        }
        
        /* 
         * Regist to DMA Tx Block 
         */
        /* Set this DMA Tx Block Status to "Requested". */
        pHandle->sDMATxBlock[ pHandle->nTxBlockCount ].nStatus = FBOOT_STATUS_REQUESTED;

        /* Set DMA Transfer size. (Maximum size is FBOOT_DMA_SIZE.) */
        if(nRemainedSize < FBOOT_DMA_SIZE){
            pHandle->sDMATxBlock[ pHandle->nTxBlockCount ].nSize = nRemainedSize;
        }else{
            pHandle->sDMATxBlock[ pHandle->nTxBlockCount ].nSize = FBOOT_DMA_SIZE;
        }
        
        /* Set source and destination address. */
        pAbsFlashAddr
            = FBOOT_ConvertToAbsFlashAddr(pHandle->hFlashHandle, 
                                        ((uint32_t)pSrcAddr + (nSize-nRemainedSize)));
        if(pAbsFlashAddr==NULL){
            /*lint -save -e534 */
            FAPI_SYS_PRINT_MSG("Failed on coverting to flash absolute address.\n");
            /*lint -restore */
            retVal = FAPI_ERR_BAD_PARAMETER;
            break;
        }
        pHandle->sDMATxBlock[ pHandle->nTxBlockCount ].pSrcAddr = pAbsFlashAddr;
        pHandle->sDMATxBlock[ pHandle->nTxBlockCount ].pDstAddr 
                                    =(void*)((uint32_t)pDstAddr + (nSize-nRemainedSize));

        /* Set nTxID for this transmission. */
        pHandle->sDMATxBlock[ pHandle->nTxBlockCount ].nTxID = nTxID;

        /* Update remained size. (for devided data. */
        nRemainedSize -= pHandle->sDMATxBlock[ pHandle->nTxBlockCount ].nSize;


        /* if unused DMA found, request to start DMA. */
        if((pHandle->nTxBlockCount) < FBOOT_DMA_NUM ){
            nNo = (pHandle->nTxBlockCount);
            
            pHandle->pDMARequest[nNo]->optHandle = (FAPI_SYS_HandleT)(pHandle->nTxBlockCount);
            pHandle->pDMARequest[nNo]->channelConfig.chLength
                            = pHandle->sDMATxBlock[pHandle->nTxBlockCount].nSize;
            pHandle->pDMARequest[nNo]->channelConfig.chRdaddr_OffsetAddress 
                            = (uint32_t) (pHandle->sDMATxBlock[pHandle->nTxBlockCount].pSrcAddr);
            pHandle->pDMARequest[nNo]->channelConfig.chWraddr_OffsetAddress
                            = (uint32_t) (pHandle->sDMATxBlock[pHandle->nTxBlockCount].pDstAddr);
                                                                
            /* DMA start request */
            if( FAPI_DMA_SendRequest( pHandle->pDMARequest[nNo] ) == FAPI_OK )
            {
                /* Change this DMA Tx Block status to "Executiong". */
                pHandle->sDMATxBlock[ pHandle->nTxBlockCount ].nStatus =FBOOT_STATUS_EXECUTING;
            }else{
                /* Failed on DMA start request */ 
                FAPI_DMA_ReleaseRequest( pHandle->pDMARequest[nNo] );
                
                pHandle->sDMATxBlock[ pHandle->nTxBlockCount ].nStatus =FBOOT_STATUS_ERROR;
                /*lint -save -e534 */
                FAPI_SYS_PRINT_MSG("Failed on DMA start request \r\n");
                /*lint -restore */
                retVal=FAPI_ERR_DEVICE_BUSY;
                break;
            }
        }

        /* Countup DMA Tx Block count */
        pHandle->nTxBlockCount++;
        
        /* Countup DMA Tx Image count */
        pHandle->sDMATxImage[nTxID].nTotal_TxBlock++;
        
    }
    if(retVal==FAPI_OK){
        return (int32_t)nTxID;
    }
    else{
        return retVal;
    }
}


/*!
*******************************************************************************
** 
** \brief  Wait for DMA Transmission is completed.
** 
** \param  pHandle       Fast Bootloader handle.
** 
** \return 
** - #FAPI_OK  if transration completed.
** - #FBOOT_DMA_TXID_MAX If no DMA transmission requested.
** - #FAPI_ERR_BAD_PARAMETER if a given parameter was invalid.
** - #FAPI_ERR_DEVICE_BUSY if DMA transmission failed.
******************************************************************************/
int32_t FBOOT_WaitBurstTransmission (FBOOT_HandleT* pHandle)
{
    int32_t retVal;
    uint32_t nTxID;
    uint32_t nRemaindRequest;
    uint32_t loop=1;

    /* Check parameter */
    if( pHandle==NULL){
        /*lint -save -e534 */
        FAPI_SYS_PRINT_MSG("WaitBurstTransmission: Bad parameter.\r\n");
        /*lint -restore */

        return FAPI_ERR_BAD_PARAMETER;
    }
    
    retVal=FBOOT_DMA_TXID_MAX;
    
    while(loop){
        nRemaindRequest=0;
        for(nTxID=0; nTxID<FBOOT_IMAGE_NUM_MAX; nTxID++){
            /* Skip unrequested image check */
            if( (pHandle->sDMATxImage[nTxID].nTotal_TxBlock) == 0 ){
                continue;
            }
            
            /* (1) If transmission completed image found. */
            if( (pHandle->sDMATxImage[nTxID].nComplete_TxBlock) 
                    == (pHandle->sDMATxImage[nTxID].nTotal_TxBlock) )
            {
                /* Clear request of this image*/
                (pHandle->sDMATxImage[nTxID].nTotal_TxBlock) = 0;
                
                /* This image transmission completed.*/
                retVal=(int32_t)nTxID;
                break;
            }
            
            /* (2) If error has reported from DMA Callback. */
            if( (pHandle->sDMATxImage[nTxID].nTotal_TxBlock) == -1){
                retVal=FAPI_ERR_DEVICE_BUSY;
                /*lint -save -e534 */
                FAPI_SYS_PRINT_MSG("found error DMA TxBlock.\r\n");
                /*lint -restore */
                break;
            }
            
            /*(3) Calculate the number of under-transmission image.*/
            nRemaindRequest++;
        }
        
        /* Exit this wait loop on following case.
           (1) Found transfered Image found. (normal case)
           (2) Found error report from DMA callback. (error case)
           (3) No under-transmission image found.  (normal case) */
        if( (retVal != FBOOT_DMA_TXID_MAX) || (nRemaindRequest == 0) ){
            /* Found transfered image or error happend. So, exit this loop. */
            loop=0;
        }
        else {
            /* Wait for transfer completion. DMA Callback function will wake-up.*/
            
            /* sleeping... */
            (void)FAPI_SYS_GET_SEMAPHORE( pHandle->hSemaphore, FAPI_SYS_SUSPEND );
            
            /* wake-upped. Go loop-head to check image from first. */
        }
    }/* End of while loop */
    
    return retVal;
}


/*!
*******************************************************************************
** 
** \brief  Cleanup resources for DMA Transmission.
** 
** \param  pHandle       Fast Bootloader handle.
** 
** \return 
** - #FAPI_OK  if transration completed.
** - #FAPI_ERR_BAD_PARAMETER if a given parameter was invalid.
******************************************************************************/
int32_t FBOOT_CleanupBurstTransmission (FBOOT_HandleT* pHandle)
{
    uint32_t i;
    
    /* Check parameter */
    if( pHandle==NULL){
        /*lint -save -e534 */
        FAPI_SYS_PRINT_MSG("CleanupBurstTransmission: Bad parameter.\r\n");
        /*lint -restore */

        return FAPI_ERR_BAD_PARAMETER;
    }
    
    /* Wait all DMA transfer completed or terminated by error. */

    for(i=0; i< FBOOT_DMA_NUM; i++){
        if(pHandle->sDMATxBlock[i].nStatus ==FBOOT_STATUS_EXECUTING){
            (void)FAPI_DMA_CancelRequest(pHandle->pDMARequest[i], FAPI_DMA_FEATURE_SDRAM_COPY);
        }
    }

    return FAPI_OK;
}


/*!
*******************************************************************************
** 
** \brief  DMA Callback function
** 
** \return -
******************************************************************************/
void    FBOOT_DMA_Callback (FAPI_DMA_RequestT *pDMARequest)
{
    uint32_t i;
    FBOOT_HandleT* pHandle;
    uint32_t nDMATxBlockNum;
    int32_t retVal;
    uint32_t nTxID;
    uint32_t bErrorFlag = FBOOT_FALSE ;
    
    /* Check parameter */
    if( pDMARequest==NULL){
        
        /*lint -save -e534 */
        FAPI_SYS_PRINT_MSG(" DMA Callback passed NULL argument. \r\n");
        /*lint -restore */
        return;
    }
    
    pHandle = (FBOOT_HandleT* ) pDMARequest->optData;
    nDMATxBlockNum = (uint32_t) pDMARequest->optHandle;
    nTxID = pHandle->sDMATxBlock[nDMATxBlockNum].nTxID;

    /* Check DMA Status */
    if( pDMARequest->result <0 /*== FAPI_DMA_REQUEST_STATUS_ABORTED*/){
        pHandle->sDMATxBlock[nDMATxBlockNum].nStatus = FBOOT_STATUS_ERROR;
        pHandle->sDMATxImage[nTxID].nTotal_TxBlock = -1;
        bErrorFlag = FBOOT_TRUE;

        /*lint -save -e534 */
        FAPI_SYS_PRINT_MSG("DMA Failed src: 0x%08x dst: 0x%08x size:%d \r\n", 
                            pHandle->sDMATxBlock[nDMATxBlockNum].pSrcAddr,
                            pHandle->sDMATxBlock[nDMATxBlockNum].pDstAddr,
                            pHandle->sDMATxBlock[nDMATxBlockNum].nSize);
        /*lint -restore */
    }
    else{
        pHandle->sDMATxBlock[nDMATxBlockNum].nStatus = FBOOT_STATUS_COMPLETED;
        pHandle->sDMATxImage[nTxID].nComplete_TxBlock++;

        /*lint -save -e534 */
        FAPI_SYS_PRINT_DEBUG(2, "DMA Complete src: 0x%08x dst: 0x%08x size:%d \r\n", 
                            pHandle->sDMATxBlock[nDMATxBlockNum].pSrcAddr,
                            pHandle->sDMATxBlock[nDMATxBlockNum].pDstAddr,
                            pHandle->sDMATxBlock[nDMATxBlockNum].nSize);
        /*lint -restore */
    }
    
    if( ((pHandle->sDMATxImage[nTxID].nTotal_TxBlock) 
            == (pHandle->sDMATxImage[nTxID].nComplete_TxBlock))
        ||  (bErrorFlag==FBOOT_TRUE) )
    {
        /* Wake up app which is waitting on WaitBurstTransmission(). */
        FAPI_SYS_SET_SEMAPHORE(pHandle->hSemaphore, RTOS_NO_SUSPEND);
    }

    /* On error case return */
    if( bErrorFlag == FBOOT_TRUE){
        return;
    }

    /* Search for untransfered request and start DMA transmission. */
    for( i=(nDMATxBlockNum+1); i<(pHandle->nTxBlockCount); i++){
        /* Found untransfered request. */
        if( pHandle->sDMATxBlock[i].nStatus ==FBOOT_STATUS_REQUESTED){
                /* Set source/destination address*/
                pDMARequest->channelConfig.chRdaddr_OffsetAddress 
                            = (uint32_t) pHandle->sDMATxBlock[i].pSrcAddr;
                            
                pDMARequest->channelConfig.chWraddr_OffsetAddress 
                            = (uint32_t) pHandle->sDMATxBlock[i].pDstAddr;
                pDMARequest->channelConfig.chLength     
                            = pHandle->sDMATxBlock[i].nSize;
                pDMARequest->optHandle = (FAPI_SYS_HandleT)i;


                /* DMA start request */
                retVal = FAPI_DMA_SendRequest( pDMARequest );
                if( retVal == FAPI_OK )
                {
                    /* Change this DMA Tx Block status to "Executiong". */
                    pHandle->sDMATxBlock[i].nStatus  = FBOOT_STATUS_EXECUTING;
                }else{
                    /* Failed on DMA start request */ 
                    FAPI_DMA_ReleaseRequest( pDMARequest );
                    pHandle->sDMATxBlock[i].nStatus = FBOOT_STATUS_ERROR;
                    /*lint -save -e534 */
                    FAPI_SYS_PRINT_MSG("DMACallback Failed on DMA start request ret:%d no:%d\r\n", 
                            retVal, i);
                    /*lint -restore */
                }
            break;
        }
        /* If unused DMA Tx Block found, no more untransfered requet exists.
           So, exit this search loop. */
        if(pHandle->sDMATxBlock[i].nStatus ==FBOOT_STATUS_UNUSED){
            break;
        }
    }
    
    return;
}

/*!
*******************************************************************************
** 
** \brief  Choose slot images to boot.
** 
** \param  hFlashHandle  FlashRom driver handle
** \param  pSlotHeader   Selected slot header info will be stored.
** 
** \return 
** - #  !NULL            Slected slot header address.
** - #  NULL             Not found valid slot.
******************************************************************************/
void * FBOOT_ChooseSlot(FAPI_SYS_HandleT hFlashHandle , BOOTER_FlashSlotHeaderT* pSlotHeader)
{
    BOOTER_FlashSlotHeaderT sSlotHeader[FBOOT_SLOT_NUM];
    uint32_t i;
#if (FBOOT_ENABLE_CRC_CHECK)
    int32_t retVal;
#endif /* (FBOOT_ENABLE_CRC_CHECK) */
    uint32_t nLatestVersion = 0;
    void *   pLatestSlotHeaderAddr =NULL;
    void *   pSlotHeaderAddr;
#if (FBOOT_FAPI_V12)
    FAPI_BOOT_InfoT* pFapiBootInfo  = FAPI_BOOT_INFO_PTR;
#else  /* #if (FBOOT_FAPI_V12) */
    FAPI_BOOT_InfoT* pFapiBootInfo  = (FAPI_BOOT_InfoT*)BOOTER_SYMBOLS_PTR;
#endif /* #if (FBOOT_FAPI_V12) */

    /* Setup booter symbols */
    pFapiBootInfo->slotIndex = 0xFFFFFFFFUL;
    pFapiBootInfo->bootMode = FAPI_BOOT_MODE_BOOTER; 
    pFapiBootInfo->bootDevice =FAPI_BOOT_DEVICE_PFLASH;

    /* Check parameter*/
    if( (hFlashHandle == 0) || (pSlotHeader==NULL)){
        /*lint -save -e534 */
        FAPI_SYS_PRINT_MSG("ChooseSlot: Bad parameter.\r\n");
        /*lint -restore */

        return NULL;
    }
    
    for( i=0;i<FBOOT_SLOT_NUM;i++){
        memset( &sSlotHeader[i], 0, sizeof(BOOTER_FlashSlotHeaderT));
        sSlotHeader[i].status = 0xFFFFFFFF;

        pSlotHeaderAddr = (void*)(FBOOT_SLOT_OFFSET + (FBOOT_SLOT_SIZE * i));
        (void)FAPI_FLASH_ReadBytes( hFlashHandle, 
                              (uint32_t)pSlotHeaderAddr, 
                              sizeof(BOOTER_FlashSlotHeaderT), 
                              &sSlotHeader[i]);
        /* Check slog signature. */
        if ( (sSlotHeader[i].signature == BOOTER_SLOTHDR_SIG) &&
            ( sSlotHeader[i].status  == 0xFFFFFFFE ))
        {

            if(nLatestVersion <  sSlotHeader[i].version){
#if (FBOOT_ENABLE_CRC_CHECK)
                /* check slot Crc.  */
                retVal = FBOOT_CheckSlot( hFlashHandle, &sSlotHeader[i], 
                            (uint32_t)pSlotHeaderAddr ); 
                if(retVal == FBOOT_TRUE)
#endif /* (FBOOT_ENABLE_CRC_CHECK) */
                {
                    nLatestVersion = sSlotHeader[i].version;
                    pLatestSlotHeaderAddr = pSlotHeaderAddr;
                    memcpy( pSlotHeader, &sSlotHeader[i], sizeof(BOOTER_FlashSlotHeaderT)); 
                    
                    /* update slotIndex of booter symbol */
                    pFapiBootInfo->slotIndex = i;
                }
            }
            /* store slot address information in the memory.  */
            switch(i){
                case 0:
                    pFapiBootInfo->slotAddress0 = (uint32_t)pSlotHeaderAddr;
                    break;
                case 1:
                    pFapiBootInfo->slotAddress1 = (uint32_t)pSlotHeaderAddr;
                    break;
                default:
                    break;
            }
        }
        
    }

    return pLatestSlotHeaderAddr;
}


/*!
*******************************************************************************
** 
** \brief  Perform CRC check on given slot (payload + part of slot header).
** 
** \param  hFlashHandle   FlashRom driver handle
** \param  pSlotHeader   array of slot header structures
** \param  slotAddr      byte address relative to flash start
** 
** \return
** - #  FBOOT_TRUE       If slot is valid.
** - #  FBOOT_FALSE      If slot is invalid.
** - #  -1               If error happend.
******************************************************************************/
#if (FBOOT_ENABLE_CRC_CHECK)
static int32_t FBOOT_CheckSlot(FAPI_SYS_HandleT hFlashHandle, const BOOTER_FlashSlotHeaderT* pSlotHeader, uint32_t nSlotAddr )
{
    uint32_t crc;
    uint32_t nCrcOffset;
    uint32_t nCalcSize;
    uint32_t nSlotSize;
    
    if( (hFlashHandle==0) || (pSlotHeader ==NULL)){
        /*lint -save -e534 */
        FAPI_SYS_PRINT_MSG("CheckSlot: Bad parameter.\r\n");
        /*lint -restore */

        return -1;
    }
    
    nCrcOffset = sizeof(uint32_t) * 3;
    nSlotSize  = (pSlotHeader->slotSize)*sizeof(uint32_t);
    if(nSlotSize > nCrcOffset){
        nCalcSize  = nSlotSize - nCrcOffset;
    }else{
        /*lint -save -e534 */
        FAPI_SYS_PRINT_MSG("Invalid CRC Offset%d.\r\n",nCrcOffset );
        /*lint -restore */
        return -1;
    }
    
    /* calculate checksum (skip slot header elements up to crc field) */
    crc = FBOOT_CalcCrcFlash (hFlashHandle, nSlotAddr + nCrcOffset,nCalcSize); 
    
    if ( crc != pSlotHeader->crc32 ){ 
        return FBOOT_FALSE;
    }
    else{                      
        return FBOOT_TRUE;
    }
}
#endif /* FBOOT_ENABLE_CRC_CHECK */


/*!
*******************************************************************************
** 
** \brief  Calclate flash absolute address. 
** 
** \param  hFapiFlashHandle      Flash driver handle.
** \param  nRelAddr              FlashRom offset address.
** 
** \return
** -  !NULL    FlashROM absolute address
** -  NULL     if error happen.
******************************************************************************/
static void * FBOOT_ConvertToAbsFlashAddr(FAPI_SYS_HandleT hFapiFlashHandle, uint32_t nRelAddr)
{
    flashHandleT *pFlashHandle;
    uint32_t nRatio, nAbsAddr;  
    

    /* Check parameters */
    pFlashHandle = (flashHandleT*)hFapiFlashHandle;
    if( pFlashHandle==NULL )
    {
        /*lint -save -e534 */
        FAPI_SYS_PRINT_MSG("ConvertToAbsFlashAddr: Bad parameter(NULL Pointer)\r\n");
        /*lint -restore */
        
        return NULL;
    }

    if( ((pFlashHandle->blockPtr->devicePtr->descriptorPtr->portBits)==0) ||
        ((pFlashHandle->blockPtr->devicePtr->descriptorPtr->dataBits)==0) )
    {
        /*lint -save -e534 */
        FAPI_SYS_PRINT_MSG("ConvertToAbsFlashAddr: Bad parameter port=%u data=%u .\r\n",
                            (pFlashHandle->blockPtr->devicePtr->descriptorPtr->portBits),
                            (pFlashHandle->blockPtr->devicePtr->descriptorPtr->dataBits));
        /*lint -restore */
        
        return NULL;
    }


  /*****************************************************************************************
   * ----------------------------------------                                              *
   * Flash access original code (summary)                                                  *
   * ----------------------------------------                                              *
   * int32_t nSectorIndex, nSectorStartAddr;                                               *
   *                                                                                       *
   * nSectorIndex= FAPI_FLASH_GetSectorIndex(hFapiFlashHandle, nRelAddr);                  *
   * nSectorStartAddr= FAPI_FLASH_GetSectorStart(hFapiFlashHandle,(uint32_t)nSectorIndex );*
   * nRatio = (pFlashHandle->blockPtr->devicePtr->descriptorPtr->portBits)                 *
   *           /(pFlashHandle->blockPtr->devicePtr->descriptorPtr->dataBits);              *
   * nAbsAddr = nRatio * (uint32_t)nSectorStartAddr;                                       *
   * nAbsAddr += nRatio * (nRelAddr - (uint32_t)nSectorStartAddr);                         *
   * nAbsAddr +=  FIO_ADDRESS(UPI,FBOOT_PFLASH_ADDR);                                       *
   *****************************************************************************************/

    nRatio = (pFlashHandle->blockPtr->devicePtr->descriptorPtr->portBits)
                /(pFlashHandle->blockPtr->devicePtr->descriptorPtr->dataBits);
    nAbsAddr = (nRatio * nRelAddr) + FIO_ADDRESS(UPI,FBOOT_PFLASH_ADDR);


    return (void*)nAbsAddr;
}
