/*!
*******************************************************************************
**
** \file        ./applications/fastboot/src/fastboot.h
** 
** \version     $Id$
**
** \author      
**
** \brief       Bootloader application 'fastboot'
**
** \attention   THIS SAMPLE CODE IS PROVIDED AS IS. FUJITSU MICROELECTRONICS
**              ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR
**              OMMISSIONS.
**
** (C) Copyright 2008-2010 by Fujitsu Microelectronics Europe GmbH
** (C) Copyright 2009-2010 by Fujitsu Microelectronics Limited
*******************************************************************************
*/

#ifndef __FASTBOOT_H__
#define __FASTBOOT_H__

#define FBOOT_FAPI_V12   1

#include <boot/booterlib.h>
#include <fapi/sys_driver.h>
#include <fapi/drv_dma.h>
#include <fapi/drv_boot.h>
#include <rtos/rtos.h>

#if (FBOOT_FAPI_V12)
#include <boot/rom_symbols.h>
#else  /* #if (FBOOT_FAPI_V12) */
#include <boot/booter_symbols.h>
#endif /* #if (FBOOT_FAPI_V12) */

/*---------------------------------------------------------------------------*/
/* Boolean definition                                                        */
/*---------------------------------------------------------------------------*/
#define FBOOT_TRUE                  1
#define FBOOT_FALSE                 0


/*---------------------------------------------------------------------------*/
/* Status definition                                                         */
/*---------------------------------------------------------------------------*/
#define FBOOT_STATUS_UNUSED         0
#define FBOOT_STATUS_REQUESTED      1
#define FBOOT_STATUS_EXECUTING      2
#define FBOOT_STATUS_COMPLETED      3
#define FBOOT_STATUS_ERROR          -1

/*---------------------------------------------------------------------------*/
/* Slot settings                                                             */
/*---------------------------------------------------------------------------*/
#define FBOOT_SLOT_NUM      BOOTER_NUM_SLOTS
#define FBOOT_SLOT_OFFSET   (1*1024*1024+0x40000)
#define FBOOT_SLOT_SIZE     (7*1024*1024)

/*---------------------------------------------------------------------------*/
/* Slot CRC Check option                                                     */
/*---------------------------------------------------------------------------*/
/* If CRC cheched enabled (=1), it may take a few seconds more. */
#define FBOOT_ENABLE_CRC_CHECK      0

/*---------------------------------------------------------------------------*/
/* Image settings                                                            */
/*---------------------------------------------------------------------------*/
 /* maximum number of image included in one slot. */
#define FBOOT_IMAGE_NUM_MAX        BOOTER_MAX_IMAGE_COUNT


/*---------------------------------------------------------------------------*/
/* DMA settings                                                              */
/*---------------------------------------------------------------------------*/
/* Number of DMA to be used. If zero specified, transfered by PIO. */
#define FBOOT_DMA_NUM               6  

/* DMA maximum size is 2046byte. ( line_length(2byte) x line_num(x03ff) ) */
#define FBOOT_DMA_SIZE              (2046)

/* This value must be grater than (FBOOT_SLOT_SIZE / FBOOT_DMA_SIZE ) */ 
#define FBOOT_DTB_NUM               (4*1024)

/* Do not change */
#define FBOOT_DMA_TXID_MAX          FBOOT_IMAGE_NUM_MAX


/*---------------------------------------------------------------------------*/
/* IO_Address                                                                */
/*---------------------------------------------------------------------------*/
#ifndef FIO_ADDRESS
#define FIO_ADDRESS(block,address) (address)
#endif /*#ifdef FIO_ADDRESS*/
#define FBOOT_PFLASH_ADDR 0x80000000

/*!
****************************************************************************
** \brief DMA Transfer Block management data structure
**
** This data structure is used on DMA transfer functions to manage each DMA 
** transmissions.
****************************************************************************
*/
typedef struct
{
    int32_t     nStatus;
    void *      pSrcAddr;
    void *      pDstAddr;
    uint32_t    nSize;
    uint32_t    nTxID;
}
FBOOT_DMA_TxBlockT;



/*!
****************************************************************************
** \brief Transfer Image management data structure
**
** This data structure is used on DMA transfer functions to manage image.
****************************************************************************
*/
typedef struct
{
    int32_t    nTotal_TxBlock;
    int32_t    nComplete_TxBlock;
}
FBOOT_DMA_TxImageT;

/*!
****************************************************************************
** \brief FastBoot Handle
**
** This is Fast Boot handle
****************************************************************************
*/
typedef struct
{
#if FBOOT_DMA_NUM
    FAPI_DMA_RequestT*  pDMARequest[FBOOT_DMA_NUM];
#else
    FAPI_DMA_RequestT*  pDMARequest[1];
#endif

    FAPI_SYS_SemaphoreT hSemaphore;
    FAPI_SYS_HandleT    hUartHandle;
    FAPI_SYS_HandleT    hFlashHandle;
    FBOOT_DMA_TxBlockT  sDMATxBlock[FBOOT_DTB_NUM];
    FBOOT_DMA_TxImageT  sDMATxImage[FBOOT_IMAGE_NUM_MAX];
    uint32_t            nTxBlockCount;
}
FBOOT_HandleT;



/*---------------------------------------------------------------------------*/
/* Functions                                                                 */
/*---------------------------------------------------------------------------*/
int32_t FBOOT_ExecutePioTransmission (FBOOT_HandleT*, void*, void*, uint32_t);
int32_t FBOOT_InitBurstTransmission (FBOOT_HandleT*);
int32_t FBOOT_DeinitBurstTransmission (FBOOT_HandleT*);
int32_t FBOOT_ExecuteBurstTransmission (FBOOT_HandleT*, void*, void*, uint32_t);
int32_t FBOOT_WaitBurstTransmission (FBOOT_HandleT*);
int32_t FBOOT_CleanupBurstTransmission (FBOOT_HandleT*);
void    FBOOT_DMA_Callback (FAPI_DMA_RequestT *pDMARequest);
void*   FBOOT_ChooseSlot(FAPI_SYS_HandleT , BOOTER_FlashSlotHeaderT* );
static int32_t FBOOT_CheckSlot(FAPI_SYS_HandleT, const BOOTER_FlashSlotHeaderT*, uint32_t);
uint32_t FBOOT_Inflate( uint8_t*, uint32_t, uint8_t*, uint32_t);
static void * FBOOT_ConvertToAbsFlashAddr(FAPI_SYS_HandleT, uint32_t);
#endif  /* ifdef __FASTBOOT_H__ */
